from typing import Optional, Any, Dict
from ._DataType import DataType


class Message(object):
    def __init__(self, token, data_type=DataType.empty, data=None, timeout=-1, headers=None):
        # type: (int, int, Optional[Any], int, Optional[Dict[str, str]]) -> None
        self.token = token
        self.data_type = data_type
        self.data = data
        self.timeout = timeout
        self.headers = headers if headers is not None else {}
        self.imp_message = None  # type: Any

    def __eq__(self, o):
        # type: (object) -> bool
        if not isinstance(o, Message):
            return False

        return \
            self.token == o.token \
            and self.data_type == o.data_type \
            and self.data == o.data \
            and self.timeout == o.timeout \
            and self.headers == o.headers

    def __ne__(self, o):
        # type: (object) -> bool
        return not self.__eq__(o)

    def __str__(self):
        return str({
            "token": self.token,
            "data_type": self.data_type,
            "timeout": self.timeout,
            "headers": self.headers
        })



from ._StartServiceInteractor import StartServiceInteractor
from ._CreateTableOrderInteractor import CreateTableOrderInteractor
from ._ReopenTableInteractor import ReopenTableInteractor
from ._CloseTableInteractor import CloseTableInteractor
from ._SetTableAvailableInteractor import SetTableAvailableInteractor
from ._DoChangeTipInteractor import DoChangeTipInteractor
from ._ChooseLineSeatInteractor import ChooseLineSeatInteractor
from ._NCouponInteractor import NCouponInteractor
from ._NDiscountInteractor import NDiscountInteractor
from ._ServicePeriodInteractor import ServicePeriodInteractor
from ._OpenDayInteractor import OpenDayInteractor
from ._OrderInteractor import OrderInteractor
from ._DailyGoalsInteractor import DailyGoalsInteractor
from ._MessageDialogInteractor import MessageDialogInteractor
from ._UserInteractor import UserInteractor
from ._TabsInteractor import TabsInteractor
from ._FiscalWrapperInteractor import FiscalWrapperInteractor
from ._SectorOperatorInteractor import SectorOperatorInteractor
from ._TableInteractor import TableInteractor
from ._LoyaltyInteractor import LoyaltyInteractor
from ._MaintenanceInteractor import MaintenanceInteractor
from ._PrinterInteractor import PrinterInteractor

__all__ = [
    "StartServiceInteractor",
    "CreateTableOrderInteractor",
    "ReopenTableInteractor",
    "CloseTableInteractor",
    "SetTableAvailableInteractor",
    "DoChangeTipInteractor",
    "ChooseLineSeatInteractor",
    "NCouponInteractor",
    "NDiscountInteractor",
    "ServicePeriodInteractor",
    "OpenDayInteractor",
    "OrderInteractor",
    "DailyGoalsInteractor",
    "MessageDialogInteractor",
    "UserInteractor",
    "FiscalWrapperInteractor",
    "SectorOperatorInteractor",
    "TableInteractor",
    "TabsInteractor",
    "LoyaltyInteractor",
    "MaintenanceInteractor",
    "PrinterInteractor",
]



from typing import Optional

from posot import OrderTaker
from sysactions import (
    get_model,
    get_posot,
)

from application.model.order import Order


class VerifyAndRemoveBenefitsUnitOfWork(object):
    def __init__(
        self,
        pos_id,         # type: int
        order,          # type: Order
        pos_ot=None,    # type: Optional[OrderTaker]
    ):
        # type: (...) -> None

        self.pos_id = pos_id
        self.order = order
        self.model = None
        self._pos_ot = pos_ot

    @property
    def pos_ot(
        self,
    ):
        # type: (...) -> OrderTaker

        if self._pos_ot is not None:
            return self._pos_ot

        if self.model is None:
            self.model = get_model(posid=self.pos_id)
        return get_posot(model=self.model)



from basereport import SimpleReport
from report import Presenter, Part
from report.command import BoldCommand, NewLineCommand, AlignCommand


class SampleReportPresenter(Presenter):

    def present(self, dto):
        parts = [
            Part("$OPERATOR"),
            Part(None, [NewLineCommand()]),
            Part("$OPERATOR"),
            Part(None, [BoldCommand(True)]),
            Part("$TABLE $SIGN_IN"),
            Part(None, [BoldCommand(False)]),
            Part("$OPERATOR"),
            Part(None, [NewLineCommand()]),

            Part("$OPERATOR"),
            Part(None, [BoldCommand(True)]),
            Part("$TABLE $SIGN_IN"),
            Part(None, [NewLineCommand()]),
            Part("$TABLE $SIGN_IN "), Part("$TABLE $SIGN_IN"),
            Part(None, [NewLineCommand()]),
            Part("$OPERATOR"),
            Part(None, [BoldCommand(False)]),
            Part(" $OPERATOR"),
            Part(None, [NewLineCommand()]),


            Part(None, [AlignCommand(AlignCommand.Alignment.left)]),
            Part("$TABLE $SIGN_IN"), Part("$ORDER $OPERATOR"),
            Part(None, [NewLineCommand()]),
            Part(None, [AlignCommand(AlignCommand.Alignment.center)]),
            Part("$TABLE $SIGN_IN"), Part("$ORDER $OPERATOR"),
            Part(None, [NewLineCommand()]),
            Part(None, [AlignCommand(AlignCommand.Alignment.right)]),
            Part("$TABLE $SIGN_IN"), Part("$ORDER $OPERATOR"),
            Part(None, [NewLineCommand()])
        ]
        return SimpleReport(parts, 38)



from typing import Dict


class CustomImages(object):
    def __init__(
        self,
        custom_images,    # type: Dict
    ):
        # type: (...) -> CustomImages

        self.custom_images = custom_images



# encoding: utf-8
import locale

from typing import (
    Any,
    List,
    Optional,
    Union,
)

import constants
from basereport import (
    TableReport,
    ReportColumnDefinition,
)
from report import Part
from helper import remove_accents

COLS = 38
SINGLE_SEPARATOR = ("-" * COLS)
SEPARATOR = ("=" * COLS)
DESCR = 17
DATE_TIME_FMT = "%d/%m/%Y %H:%M:%S"


def center(s):
    s = _cut(s)
    miss = COLS - len(s)
    if miss == 0:
        return s
    left = miss / 2
    return (" " * left) + s + "\n"


def join(dic1, dic2):
    d = dict(dic1)
    d.update(dic2)
    return d


def single_line_separator():
    return SINGLE_SEPARATOR + "\n"


def double_line_separator():
    return SEPARATOR + "\n"


def empty_line():
    return " " * COLS + "\n"


def break_line():
    return "\n"


def add_amount_line(descr, qty=None, amount=None, info=None):
    if amount is not None:
        amount = format_number(float(amount))

    descr = descr[0:DESCR]
    if len(descr) < DESCR:
        descr += '.' * (DESCR - len(descr))

    if qty is not None:
        line = "%s: [%4s] " % (descr, qty)
    else:
        line = "%s: " % descr

    if amount is not None:
        if qty is None:
            line += "       "
        remain = COLS - len(line) - 3
        line += "R$ %*s" % (remain, amount)
    elif info is not None:
        remain = COLS - len(line)
        line += "%*.*s" % (remain, remain, info)

    line += "\n"

    return line


def add_table_line(columns, columns_height, align=None, format_type=None, character_to_add=None):
    index = 0
    lines = ""
    total_columns = len(columns)
    for column in columns:
        current_format_type = 'string' if format_type is None else format_type[index]
        current_align = 'left' if align is None else align[index]
        current_character_to_add = ' ' if character_to_add is None else character_to_add[index]
        current_final_character = ''
        if len(current_character_to_add) > 1:
            current_character_to_add = character_to_add[0][0]
            current_final_character = character_to_add[0][1]

        column = str(column)
        column_len = columns_height[index]

        if current_format_type == 'float':
            formatted_number = '%.2f' % float(column)
            remain = column_len - len(formatted_number)

            if current_align == 'left':
                line = '{0:<{1}s}'.format(formatted_number, remain)
            else:
                line = '{0:>{1}s}'.format(formatted_number, remain)
        elif current_format_type == 'integer':
            line = column[0:column_len]
        else:
            line = column[0:column_len]

        add_characters = column_len - len(remove_accents(line)) if column_len - len(remove_accents(line)) > 0 else 0

        if add_characters == 0:
            line = column[0:column_len - 1]
            if index != total_columns - 1:
                lines += line
                if current_final_character != '':
                    line_list = list(lines)
                    line_list[-1] = current_final_character
                    lines = "".join(line_list)

                lines += " "
            else:
                lines += line
        else:
            if current_align == 'right':
                if index != total_columns - 1:
                    lines += (current_character_to_add * (add_characters - 1)) + line + " "
                else:
                    lines += " " + (current_character_to_add * (add_characters - 1)) + line
            else:
                if index != total_columns - 1:
                    lines += line + (current_character_to_add * (add_characters - 1))

                    if current_final_character != '':
                        line_list = list(lines)
                        line_list[-1] = current_final_character
                        lines = "".join(line_list)

                    lines += " "
                else:
                    lines += line + (current_character_to_add * add_characters)

        index += 1

    return lines + "\n"


def add_key_value_line(key, value):
    line = key
    remain = COLS - len(line)
    line += value[0:remain] + "\n"

    if len(value) > remain:
        line += value[remain: (2 * COLS) - remain] + "\n"

    return line


def set_font(font_type):
    font_a = b'\x1b\x4d\x00'
    font_b = b'\x1b\x4d\x01'

    if font_type == 'B':
        return [Part(font_b, [])]

    return [Part(font_a, [])]


def bold_on():
    return [Part(b'\x1b\x45\x01', [])]


def bold_off():
    return [Part(b'\x1b\x45\x00', [])]


def cut_paper():
    return [Part(b'\x1dV\x01', [])]


def rollback_printer_default_font():
    report = "{}{}{}".format(constants.HW_INIT, constants.TXT_NORMAL, constants.TXT_BOLD_OFF)
    report += "{}{}".format(constants.TXT_FONT_A, constants.TXT_ALIGN_LT)

    return report


def _cut(s):
    return s[:COLS] if (len(s) > COLS) else s


def format_number(number, decimal_places=2, decimal_separator='.', thousand_separator=','):
    sign = '-' if number < 0 else ''
    number = abs(number)
    number, dec = ("%.*f" % (decimal_places, number)).split('.')

    if thousand_separator:
        sepPos = len(number) - 3
        while sepPos >= 1:
            number = number[:sepPos] + thousand_separator + number[sepPos:]
            sepPos -= 3

    if decimal_separator:
        number += (decimal_separator + dec)

    return sign + number


def format_line(
    title,              # type: Optional[str]
    text,               # type: Optional[str]
    columns,            # type: int
    definition,         # type: List[ReportColumnDefinition]
    encoding='ASCII',   # type: Optional[str]
):
    # type: (...) -> List

    if not text:
        return []

    lines = []
    current_string_start = 0
    current_string_end = definition[1].width if title else columns

    while len(text) > 0:
        if title and current_string_start == 0:
            lines.append(TableReport(
                lines=[[title, text[:current_string_end]]],
                column_definitions=definition,
                encoding=encoding,
            ))
        else:
            lines.append(TableReport(
                lines=[[text[current_string_start:current_string_end]]],
                column_definitions=definition,
                encoding=encoding,
            ))

        if current_string_end > len(text):
            break

        current_string_start = current_string_end
        current_string_end += columns

    return lines


def format_price(price):
    formatted_value = '{0:,.2f}'.format(price)
    return formatted_value


def str_float(
    float_num     # type: float
):
    # type: ('...') -> str
    return locale.format("%.2f", float_num, True)


def set_inline_text(
    parts,          # type: List
    title,          # type: str
    text,           # type: str
    columns,        # type: int
    definition,     # type: Any
):
    # type: (...) -> None

    value = remove_accents(
        text=text,
    )

    parts.extend(
        format_line(
            title=title,
            text=value,
            columns=columns,
            definition=definition,
        )
    )


def translate_pos_label(
    pos_id,    # type: Any[Union[str, int]]
):
    # type: (...) -> str

    from sysactions import get_model
    from helper import translate_pos_label

    pos_id = 0 if pos_id == "Todos" else pos_id
    model = get_model(posid=pos_id)
    return translate_pos_label(model=model)




class PaymentType(object):

    def __init__(
            self,
            name=None,                  # type: str
            payment_name=None,          # type: str
            payment_type=None,          # type: int
            image_path=None,            # type: str
            electronic_type_id=None     # type: int
    ):
        # type: (...) -> None

        self.name = name
        self.payment_name = payment_name
        self.payment_type = payment_type
        self.image_path = image_path
        self.electronic_type_id = electronic_type_id



import os
import json


class GetUpSellPartCodesResponse(object):
    def __init__(
            self,
            up_sell_part_codes_path  # type: str
    ):
        # type: (...) -> None

        self.up_sell_part_codes = json.loads("{}")
        if not up_sell_part_codes_path or not os.path.exists(up_sell_part_codes_path):
            return

        with open(up_sell_part_codes_path) as f:
            config_file = f.read()
            self.up_sell_part_codes = json.loads(config_file or "{}")



# -*- coding: utf-8 -*-
"""
    werkzeug.debug.repr
    ~~~~~~~~~~~~~~~~~~~

    This module implements object representations for debugging purposes.
    Unlike the default repr these reprs expose a lot more information and
    produce HTML instead of ASCII.

    Together with the CSS and JavaScript files of the debugger this gives
    a colorful and more compact output.

    :copyright: 2007 Pallets
    :license: BSD-3-Clause
"""
import codecs
import re
import sys
from collections import deque
from traceback import format_exception_only

from .._compat import integer_types
from .._compat import iteritems
from .._compat import PY2
from .._compat import string_types
from .._compat import text_type
from ..utils import escape


missing = object()
_paragraph_re = re.compile(r"(?:\r\n|\r|\n){2,}")
RegexType = type(_paragraph_re)


HELP_HTML = """\
<div class=box>
  <h3>%(title)s</h3>
  <pre class=help>%(text)s</pre>
</div>\
"""
OBJECT_DUMP_HTML = """\
<div class=box>
  <h3>%(title)s</h3>
  %(repr)s
  <table>%(items)s</table>
</div>\
"""


def debug_repr(obj):
    """Creates a debug repr of an object as HTML unicode string."""
    return DebugReprGenerator().repr(obj)


def dump(obj=missing):
    """Print the object details to stdout._write (for the interactive
    console of the web debugger.
    """
    gen = DebugReprGenerator()
    if obj is missing:
        rv = gen.dump_locals(sys._getframe(1).f_locals)
    else:
        rv = gen.dump_object(obj)
    sys.stdout._write(rv)


class _Helper(object):
    """Displays an HTML version of the normal help, for the interactive
    debugger only because it requires a patched sys.stdout.
    """

    def __repr__(self):
        return "Type help(object) for help about object."

    def __call__(self, topic=None):
        if topic is None:
            sys.stdout._write("<span class=help>%s</span>" % repr(self))
            return
        import pydoc

        pydoc.help(topic)
        rv = sys.stdout.reset()
        if isinstance(rv, bytes):
            rv = rv.decode("utf-8", "ignore")
        paragraphs = _paragraph_re.split(rv)
        if len(paragraphs) > 1:
            title = paragraphs[0]
            text = "\n\n".join(paragraphs[1:])
        else:  # pragma: no cover
            title = "Help"
            text = paragraphs[0]
        sys.stdout._write(HELP_HTML % {"title": title, "text": text})


helper = _Helper()


def _add_subclass_info(inner, obj, base):
    if isinstance(base, tuple):
        for base in base:
            if type(obj) is base:
                return inner
    elif type(obj) is base:
        return inner
    module = ""
    if obj.__class__.__module__ not in ("__builtin__", "exceptions"):
        module = '<span class="module">%s.</span>' % obj.__class__.__module__
    return "%s%s(%s)" % (module, obj.__class__.__name__, inner)


class DebugReprGenerator(object):
    def __init__(self):
        self._stack = []

    def _sequence_repr_maker(left, right, base=object(), limit=8):  # noqa: B008, B902
        def proxy(self, obj, recursive):
            if recursive:
                return _add_subclass_info(left + "..." + right, obj, base)
            buf = [left]
            have_extended_section = False
            for idx, item in enumerate(obj):
                if idx:
                    buf.append(", ")
                if idx == limit:
                    buf.append('<span class="extended">')
                    have_extended_section = True
                buf.append(self.repr(item))
            if have_extended_section:
                buf.append("</span>")
            buf.append(right)
            return _add_subclass_info(u"".join(buf), obj, base)

        return proxy

    list_repr = _sequence_repr_maker("[", "]", list)
    tuple_repr = _sequence_repr_maker("(", ")", tuple)
    set_repr = _sequence_repr_maker("set([", "])", set)
    frozenset_repr = _sequence_repr_maker("frozenset([", "])", frozenset)
    deque_repr = _sequence_repr_maker(
        '<span class="module">collections.' "</span>deque([", "])", deque
    )
    del _sequence_repr_maker

    def regex_repr(self, obj):
        pattern = repr(obj.pattern)
        if PY2:
            pattern = pattern.decode("string-escape", "ignore")
        else:
            pattern = codecs.decode(pattern, "unicode-escape", "ignore")
        if pattern[:1] == "u":
            pattern = "ur" + pattern[1:]
        else:
            pattern = "r" + pattern
        return u're.compile(<span class="string regex">%s</span>)' % pattern

    def string_repr(self, obj, limit=70):
        buf = ['<span class="string">']
        r = repr(obj)

        # shorten the repr when the hidden part would be at least 3 chars
        if len(r) - limit > 2:
            buf.extend(
                (
                    escape(r[:limit]),
                    '<span class="extended">',
                    escape(r[limit:]),
                    "</span>",
                )
            )
        else:
            buf.append(escape(r))

        buf.append("</span>")
        out = u"".join(buf)

        # if the repr looks like a standard string, add subclass info if needed
        if r[0] in "'\"" or (r[0] in "ub" and r[1] in "'\""):
            return _add_subclass_info(out, obj, (bytes, text_type))

        # otherwise, assume the repr distinguishes the subclass already
        return out

    def dict_repr(self, d, recursive, limit=5):
        if recursive:
            return _add_subclass_info(u"{...}", d, dict)
        buf = ["{"]
        have_extended_section = False
        for idx, (key, value) in enumerate(iteritems(d)):
            if idx:
                buf.append(", ")
            if idx == limit - 1:
                buf.append('<span class="extended">')
                have_extended_section = True
            buf.append(
                '<span class="pair"><span class="key">%s</span>: '
                '<span class="value">%s</span></span>'
                % (self.repr(key), self.repr(value))
            )
        if have_extended_section:
            buf.append("</span>")
        buf.append("}")
        return _add_subclass_info(u"".join(buf), d, dict)

    def object_repr(self, obj):
        r = repr(obj)
        if PY2:
            r = r.decode("utf-8", "replace")
        return u'<span class="object">%s</span>' % escape(r)

    def dispatch_repr(self, obj, recursive):
        if obj is helper:
            return u'<span class="help">%r</span>' % helper
        if isinstance(obj, (integer_types, float, complex)):
            return u'<span class="number">%r</span>' % obj
        if isinstance(obj, string_types) or isinstance(obj, bytes):
            return self.string_repr(obj)
        if isinstance(obj, RegexType):
            return self.regex_repr(obj)
        if isinstance(obj, list):
            return self.list_repr(obj, recursive)
        if isinstance(obj, tuple):
            return self.tuple_repr(obj, recursive)
        if isinstance(obj, set):
            return self.set_repr(obj, recursive)
        if isinstance(obj, frozenset):
            return self.frozenset_repr(obj, recursive)
        if isinstance(obj, dict):
            return self.dict_repr(obj, recursive)
        if deque is not None and isinstance(obj, deque):
            return self.deque_repr(obj, recursive)
        return self.object_repr(obj)

    def fallback_repr(self):
        try:
            info = "".join(format_exception_only(*sys.exc_info()[:2]))
        except Exception:  # pragma: no cover
            info = "?"
        if PY2:
            info = info.decode("utf-8", "ignore")
        return u'<span class="brokenrepr">&lt;broken repr (%s)&gt;' u"</span>" % escape(
            info.strip()
        )

    def repr(self, obj):
        recursive = False
        for item in self._stack:
            if item is obj:
                recursive = True
                break
        self._stack.append(obj)
        try:
            try:
                return self.dispatch_repr(obj, recursive)
            except Exception:
                return self.fallback_repr()
        finally:
            self._stack.pop()

    def dump_object(self, obj):
        repr = items = None
        if isinstance(obj, dict):
            title = "Contents of"
            items = []
            for key, value in iteritems(obj):
                if not isinstance(key, string_types):
                    items = None
                    break
                items.append((key, self.repr(value)))
        if items is None:
            items = []
            repr = self.repr(obj)
            for key in dir(obj):
                try:
                    items.append((key, self.repr(getattr(obj, key))))
                except Exception:
                    pass
            title = "Details for"
        title += " " + object.__repr__(obj)[1:-1]
        return self.render_object_dump(items, title, repr)

    def dump_locals(self, d):
        items = [(key, self.repr(value)) for key, value in d.items()]
        return self.render_object_dump(items, "Local variables in frame")

    def render_object_dump(self, items, title, repr=None):
        html_items = []
        for key, value in items:
            html_items.append(
                "<tr><th>%s<td><pre class=repr>%s</pre>" % (escape(key), value)
            )
        if not html_items:
            html_items.append("<tr><td><em>Nothing</em>")
        return OBJECT_DUMP_HTML % {
            "title": escape(title),
            "repr": "<pre class=repr>%s</pre>" % repr if repr else "",
            "items": "\n".join(html_items),
        }



import json
import logging

from xml.etree import cElementTree as eTree

from bustoken import TK_ELECTRONIC_PAYMENT_CANCEL_PAYMENT_ORDER
from msgbus import (
    FM_PARAM,
    TK_SYS_ACK,
)
from mw_helper import show_message_options_dialog
from sysactions import (
    get_posot,
    mbcontext,
    show_messagebox,
)
from typing import (
    Optional,
    Tuple,
)

from actions.eft import (
    get_electronic_payment_component_name,
    get_electronic_tenders,
    save_cancel_receipt,
)
from application.interactor import (
    OrderInteractor,
    PrinterInteractor,
)
from utilfunctions import is_electronic_payment

logger = logging.getLogger("EftActions")


def cancel_eft_transaction(
    pos_id,                     # type: str
    model,                      # type: eTree.XML
    order_id,                   # type: str
    tender,                     # type: eTree.XML
):
    # type: (...) -> Optional[str]

    order_taker = get_posot(model)
    order_picture = order_taker.orderPicture(
        posid=pos_id,
        orderid=order_id or ""
    )

    order_xml = eTree.XML(order_picture)
    order = order_xml.find("Order") if order_xml.find("Order") else order_xml

    operator = model.find("Operator")
    operator_id = operator.get("id", "") if operator is not None else ""

    return _cancel_eft_tender(
        pos_id=pos_id,
        order=order,
        tender=tender,
        operator_id=operator_id,
    )


def clear_multiple_tenders(
    pos_id,                 # type: str
    order,                  # type: eTree.XML
    order_id,               # type: str
    is_mobile,              # type: bool
    operator_id=None,       # type: str
):
    # type: (...) -> None

    print_customer_receipt_response = None
    tenders = get_electronic_tenders(order=order)
    order_interactor = OrderInteractor(pos_id=pos_id)
    printer_interactor = PrinterInteractor(
        pos_id=pos_id,
        mb_context=mbcontext,
    )
    for tender in tenders:
        tender_amount = float(tender.get("amount"))
        if tender_amount == 0:
            continue

        cancel_eft_data = None
        if is_electronic_payment(tender):
            cancel_eft_data = _cancel_eft_tender_with_retry(
                pos_id=pos_id,
                order=order,
                tender=tender,
                operator_id=operator_id,
            )

            if cancel_eft_data is not None:
                print_customer_receipt_response = printer_interactor.print_tef_tender_receipt(
                    is_mobile=is_mobile,
                    order=order,
                    tender_detail=json.loads(cancel_eft_data),
                    print_customer_receipt=print_customer_receipt_response,
                )

        save_cancel_receipt(
            pos_id=pos_id,
            order=order,
            tender_id=tender.get('id'),
            data=cancel_eft_data or '{}',
        )

        order_interactor.clear_order_tender(
            order_tender_id=tender.get("id"),
            order_id=order_id or order.get("orderId"),
        )

    return None


def _cancel_eft_tender(
    pos_id,             # type: str
    order,              # type: eTree.XML
    tender,             # type: eTree.XML
    operator_id,        # type: str
):
    # type: (...) -> Optional[str]

    data, tender_id = _get_cancel_eft_tender_data(
        pos_id=pos_id,
        order=order,
        tender=tender,
        operator_id=operator_id,
    )
    try:
        component_name = get_electronic_payment_component_name(pos_id=pos_id)
        if component_name is None:
            return None

        timeout = 600000000
        token = TK_ELECTRONIC_PAYMENT_CANCEL_PAYMENT_ORDER
        result = mbcontext.MB_EasySendMessage(component_name, token, FM_PARAM, data, timeout)
        if result.token == TK_SYS_ACK:
            return result.data

        logger.error("Error cancel payment: {} - Tender: {}".format(result.data, tender_id))
    except Exception as _:
        logger.exception("Error cancel payment: Tender: {}".format(tender_id))
        show_messagebox(
            posid=pos_id,
            message="$ERROR_CANCEL_ELECTRONIC_PAYMENTS_ORDER",
        )

    return None


def _cancel_eft_tender_with_retry(
    pos_id,             # type: str
    order,              # type: eTree.XML
    tender,             # type: eTree.XML
    operator_id,        # type: str
    retry_count=1,      # type: Optional[int]
):
    # type: (...) -> Optional[str]

    for attempt in range(retry_count + 1):
        cancel_response = _cancel_eft_tender(
            pos_id=pos_id,
            order=order,
            tender=tender,
            operator_id=operator_id,
        )

        if cancel_response is not None:
            return cancel_response

        if attempt >= retry_count:
            show_messagebox(
                posid=pos_id,
                message="$ERROR_CANCEL_ELECTRONIC_PAYMENTS_ORDER",
                timeout=5000,
            )
            return None

        response = show_message_options_dialog(
            pos_id=pos_id,
            buttons="$CONTINUE_REFUND|$TRY_AGAIN_REFUND",
            title="$INFORMATION",
            message="$ERROR_CANCEL_EFT_TRANSACTION_RETRY",
            timeout=60000,
        )
        if response in [0, None, '']:
            return None


def _get_cancel_eft_tender_data(
    pos_id,             # type: str
    order,              # type: eTree.XML
    tender,             # type: eTree.XML
    operator_id         # type: str
):
    # type: (...) -> Tuple[str, str]

    is_table_service = order.find("CustomOrderProperties/OrderProperty/[@key='TABLE_ID']")
    default_tender_id = tender.get("tenderId") or tender.get("id", "")
    tender_id = "" if is_table_service is None else default_tender_id
    tender_type = tender.get("type") or tender.get("tenderType")
    tender_amount = float(tender.get("amount") or tender.get("tenderAmount"))
    detail = tender.get("detail") or json.loads(tender.get("tenderDetail", "{}"))
    last_totaled = order.findall("StateHistory/State[@state='TOTALED']")[-1]
    last_totaled_date = last_totaled.get("timestamp").replace("-", "").replace(":", "")
    electronic_type = int(tender.get("electronicType") or tender.get("eletronicType"))

    data = json.dumps(
        {
            "posId": pos_id,
            "orderId": order.get("orderId"),
            "tenderId": tender_id,
            "tenderType": tender_type,
            "tenderAmount": tender_amount,
            "fiscalDate": last_totaled_date[:8],
            "fiscalTime": last_totaled_date[9:15],
            "tenderDetail": detail,
            "operatorId": operator_id or "",
            "electronicType": electronic_type
        }
    )

    return data, default_tender_id





from typing import List, Optional, Dict
from xml.etree.ElementTree import Element

from application.model.benefit import Benefit
from application.model.order import SaleLine, SaleTypes
from application.repository import NDiscountRepository
from application.service import OrderService


class Restrictions(object):
    def __init__(self, pos_id, order_picture, benefit, repository, order_service):
        # type: (int, Element, Benefit, NDiscountRepository, OrderService) -> None

        self.pos_id = pos_id
        self.order_picture = order_picture
        self.benefit = benefit
        self.repository = repository
        self.order_service = order_service
        self.price_keys = self._get_price_keys_from_benefit(self.pos_id, self.benefit, self.order_picture)
        self.exclude_products = self._get_excluded_products_from_benefit(self.benefit)
        self.valid_sale_types = self._get_sale_types(self.pos_id, self.benefit, self.order_picture)
        self.valid_brands = benefit.conditions.brands

    def _get_sale_types(self, pos_id, benefit, order_picture):
        # type: (int, Benefit, Element) -> Optional[List[str]]

        order = self.order_service.get_order(pos_id, order_picture=order_picture)
        if order.table_id is None:
            return None

        if benefit.conditions.sale_type is None:
            return None

        sale_types = []
        for sale_type in benefit.conditions.sale_type:
            sale_type = SaleTypes.get_name(sale_type)
            if sale_type:
                sale_types.append(sale_type)

        return sale_types

    def _get_price_list_from_service_period(self, service_period_ids):
        # type: (List) -> List

        if service_period_ids is None:
            return []

        price_lists = []
        services_periods = self.repository.get_all_service_periods()
        for current_id in service_period_ids:
            if current_id in services_periods:
                price_lists.append(services_periods[current_id])
        return price_lists

    def _get_price_keys_from_benefit(self, pos_id, benefit, order_picture):
        # type: (int, Benefit, Element) -> Optional[List]

        order = self.order_service.get_order(pos_id, order_picture=order_picture)
        service_period = benefit.conditions.service_period

        if not service_period:
            return None

        price_list = self._get_price_list_from_service_period(service_period)
        if not price_list:
            return []

        all_price_keys = self.repository.get_all_service_periods_price_keys()
        price_keys = []
        for sale_line in order.sale_lines:
            price_key = sale_line.price_key
            if price_key and all_price_keys[price_key] in price_list and price_key not in price_keys:
                price_keys.append(price_key)

        return price_keys

    def _get_excluded_products_from_benefit(self, benefit):
        # type: (Benefit) -> Optional[List]

        all_product_groups = self.get_products_groups()
        products = benefit.exclusions.items if benefit.exclusions.items else []
        groups = benefit.exclusions.groups if benefit.exclusions.groups else []
        for group in groups:
            if group in all_product_groups:
                products += all_product_groups[group]

        if len(products) == 0:
            return None

        return products

    def exists(self):
        # type: () -> bool

        if self.price_keys is not None or self.exclude_products or self.valid_sale_types:
            return True

        return False

    def validate(self, sale_line):
        # type: (SaleLine) -> bool

        if not self._sale_type_is_valid(sale_line):
            return False

        if not self._service_period_is_valid(sale_line):
            return False

        if self.exclude_products:
            sale_line_keys = sale_line.key.split(".")
            if any([item_key for item_key in self.exclude_products if str(item_key) in sale_line_keys]):
                return False

        if not self._brand_is_valid(sale_line):
            return False

        return True

    def get_products_groups(self):
        # type: () -> Dict

        return self.repository.get_all_product_groups()

    def _service_period_is_valid(self, sale_line):
        # type: (SaleLine) -> bool

        if not self.benefit.conditions.service_period:
            return True

        if not sale_line.service_period:
            return False

        if not sale_line.service_period.id or sale_line.service_period.id not in self.benefit.conditions.service_period:
            return False

        return True

    def _sale_type_is_valid(self, sale_line):
        # type: (SaleLine) -> bool

        if not self.valid_sale_types:
            return True

        if not sale_line.sale_type:
            return False

        if sale_line.sale_type not in self.valid_sale_types:
            return False

        return True

    def _brand_is_valid(self, sale_line):
        # type: (SaleLine) -> bool

        if not self.valid_brands:
            return True

        if not sale_line.brand:
            return False

        if sale_line.brand not in self.valid_brands:
            return False

        return True



import os
import json
from logging import Logger
from typing import Dict

from interactor.route.customization.get_custom_images.dto import GetCustomImagesRequest
from model.config import Config
from model.custom_images import CustomImages
from validation_util import validate_api_key


class GetCustomImagesInteractor(object):
    def __init__(
        self,
        logger,    # type: Logger
        config,    # type: Config
    ):
        # type: (...) -> None

        self.logger = logger
        self.config = config

    def execute(
        self,
        request,    # type: GetCustomImagesRequest
    ):
        # type: (...) -> Dict

        validate_api_key(
            request=request,
            secure_api_key_list=self.config.api_key_mapping,
        )

        if not self.config.totem_config_path or not os.path.exists(self.config.totem_config_path):
            return CustomImages({})

        with open(self.config.totem_config_path, 'rb') as f:
            data = json.load(f)

        custom_images = {}
        for custom_image in data.get('customImages'):
            key = custom_image.keys()[0]
            custom_images[key] = custom_image.get(key)
        return custom_images



from typing import List

class OrderValidationException(Exception):
    pass


class BenefitExpiredOrInactiveException(OrderValidationException):
    pass


class PodTypeNotValidException(OrderValidationException):
    pass


class InsufficientSaleAmountException(OrderValidationException):
    pass


class ErrorApplyingCoupon(OrderValidationException):
    pass


class ExcessiveSaleAmountException(OrderValidationException):
    pass


class UserLevelNotAllowedException(Exception):
    def __init__(
        self,
        min_level_required,     # type: int
    ):
        # type: (...) -> None

        self.min_level_required = min_level_required


class MissingItemsException(OrderValidationException):
    pass


class CustomBenefitAlreadyAppliedException(OrderValidationException):
    pass


class InvalidServiceTypeException(OrderValidationException):
    pass


class InvalidSaleTypeException(OrderValidationException):
    pass


class InvalidMinSeatsException(OrderValidationException):
    pass


class InvalidSectorException(OrderValidationException):
    pass


class ApplyingDiscountUnknownException(OrderValidationException):
    pass


class OrderPictureException(OrderValidationException):
    pass


class InvalidAddItemsInTotaledOrderException(OrderValidationException):
    pass


class InvalidServicePeriodOrderException(OrderValidationException):
    pass


class InvalidBrandException(OrderValidationException):
    pass


class AddItemsException(OrderValidationException):
    pass


class NeedToRemoveBenefitsPreviousBenefitsException(Exception):
    def __init__(
        self,           # type: NeedToRemoveBenefitsPreviousBenefitsException
        benefits_ids,   # type: List[str]
    ):
        # type: (...) -> None

        self.benefits_ids = benefits_ids



from domain import OrderRepository, Order, OrderTender
from old_helper import BaseRepository
from persistence import Connection

from typing import Dict, List  # noqa


class MwBusinessPeriodOrderRepository(OrderRepository, BaseRepository):
    def __init__(self, mbcontext):
        super(MwBusinessPeriodOrderRepository, self).__init__(mbcontext)

    def get_order(self, initial_date, end_date, pos, operator):
        def inner_func(conn):
            # type: (Connection) -> List[Order]
            query = self.build_query(initial_date, end_date, pos, operator)
            return self.execute_query(conn, query)

        return self.execute_with_connection(inner_func)

    @staticmethod
    def build_query(initial_date, end_date, pos, operator):
        query = MwBusinessPeriodOrderRepository.business_period_query \
            .format(initial_date.strftime("%Y%m%d"),
                    end_date.strftime("%Y%m%d"))
        if pos is not None:
            query += " and o.SessionId like '%pos={},%'".format(pos)
        if operator is not None:
            query += " and o.SessionId like '%user={},%'".format(operator)
        return query

    @staticmethod
    def execute_query(conn, query):
        order_with_payments = [(int(x.get_entry(0)),
                                int(x.get_entry(1)),
                                float(x.get_entry(2)),
                                int(x.get_entry(3)),
                                float(x.get_entry(4))) for x in conn.select(query)]

        all_orders = {}  # type: Dict[int, Order]
        for order_tuple in order_with_payments:
            order_id = order_tuple[0]
            order_state = order_tuple[1]
            order_total = order_tuple[2]
            tender_type = order_tuple[3]
            tender_value = order_tuple[4]

            if order_id in all_orders:
                order = all_orders[order_id]
            else:
                order = Order(Order.OrderSate(order_state), order_total, [])
                all_orders[order_id] = order

            order_tender = OrderTender(str(tender_type), tender_value)
            order.tenders.append(order_tender)
        return list(all_orders.values())

    business_period_query = """
select o.OrderId, o.StateId, o.TotalGross, ot.TenderId, ot.TenderAmount
from Orders o
inner join OrderTender ot
on o.OrderId = ot.OrderId
where o.BusinessPeriod >= '{0}' and o.BusinessPeriod <= '{1}' and o.StateId in (4, 5)"""



import os

from application.configurator import Configurator
from application.model.configuration import Configurations
from helper import import_pydevd
from mbcontextmessagehandler import MbContextMessageBus, MbContextMessageHandler
from messagehandlerbuilder import ComponentMessageHandlerBuilder
from msgbus import MBEasyContext


def main():
    # type: () -> None

    loader_path = os.environ["LOADERCFG"]

    import_pydevd(loader_path, 9169)

    configurator = Configurator(loader_path)
    configurations = _get_configurations(configurator)

    mb_context = MBEasyContext(configurations.service_name)
    message_bus = MbContextMessageBus(mb_context)

    message_handler_builder = ComponentMessageHandlerBuilder(
        configs=configurations,
        message_bus=message_bus,
    )

    message_handler = MbContextMessageHandler(
        message_bus,
        configurations.service_name,
        configurations.service_name,
        configurations.required_services,
        message_handler_builder
    )

    if not configurator.configs.store_id:
        configurator.get_store_id(message_bus)

    message_handler.handle_events()


def _get_configurations(configurator):
    # type: (Configurator) -> Configurations

    configurator.read_loader_configurations()
    configurator.configure_logger()

    return configurator.configs


if __name__ == "__main__":
    main()



from logging import Logger

from model.error_type import PosValidationErrorsType
from typing import Any

from interactor.route.order.add_properties.dto import AddPropertiesRequest
from model.config import Config
from model.exception import ValidationError, AuthValidationErrors
from model.order_detail import AdditionalProperty
from model.repository import PosRepositoryModel
from validation_util import \
    check_if_business_day_is_opened, \
    check_if_operator_is_logged, validate_api_key


class AddPropertiesInteractor(object):
    def __init__(self, logger, config, pos_repository):
        # type: (Logger, Config, PosRepositoryModel) -> None

        self.logger = logger
        self.config = config
        self.pos_repository = pos_repository

    def execute(self, request):
        # type: (AddPropertiesRequest) -> None

        pos_id = validate_api_key(request, self.config.api_key_mapping)

        model = self.pos_repository.get_model(pos_id)
        if not self.pos_repository.has_current_order(model):
            raise AuthValidationErrors(PosValidationErrorsType.OrderAlreadyInProgress)

        check_if_business_day_is_opened(self.pos_repository, model)
        check_if_operator_is_logged(self.pos_repository, model)

        for to_add_property in request.to_add_properties:
            self._add_property(to_add_property, pos_id, model)

    def _add_property(self, to_add_property, pos_id, model):
        # type: (AdditionalProperty, str, Any) -> None

        valid_keys = ['CUSTOMER_DOC', 'CUSTOMER_NAME', 'LOYALTY_USER_ID', 'SALE_TYPE', 'PAGER_NUMBER']

        if to_add_property.key not in valid_keys and not to_add_property.key.startswith('EXTERNAL_PROP_'):
            raise ValidationError(PosValidationErrorsType.KeyPropertyInvalid)

        pos_ot = self.pos_repository.get_pos_ot(model)

        if to_add_property.key == 'SALE_TYPE':
            if to_add_property.value not in self.config.available_sale_types:
                raise ValidationError(PosValidationErrorsType.SaleTypeInvalid)
            else:
                self.pos_repository.update_order_properties(pos_ot=pos_ot, posid=pos_id, saletype=to_add_property.value)
        else:
            self.pos_repository.add_custom_property(to_add_property.key, to_add_property.value, pos_ot)

            if to_add_property.key == "CUSTOMER_DOC":
                self.pos_repository.add_custom_property("CUSTOMER_FISCAL_DOCUMENT", to_add_property.value, pos_ot)



from messagebus import TokenCreator, TokenPriority


class NDiscountTokens(object):
    NDISCOUNT_MESSAGE_GROUP = "112"

    TK_NDISCOUNT_ADD_BENEFIT = TokenCreator.create_token(TokenPriority.low, NDISCOUNT_MESSAGE_GROUP, "1")
    TK_NDISCOUNT_REMOVE = TokenCreator.create_token(TokenPriority.low, NDISCOUNT_MESSAGE_GROUP, "2")
    TK_NDISCOUNT_APPLY_AUTOMATIC_BENEFITS = TokenCreator.create_token(TokenPriority.low, NDISCOUNT_MESSAGE_GROUP, "6")
    TK_NDISCOUNT_GET_BENEFITS = TokenCreator.create_token(TokenPriority.low, NDISCOUNT_MESSAGE_GROUP, "7")



import logging

from httpclient import HttpRequest, HttpResponse
from messagebus import HttpRequestMessage, DefaultToken
from messagehandler import MessageHandlerBuilder
from typing import Optional, Any, Dict, List

from ._GetReplyMessageBus import GetReplyMessageBus
from ._InvalidResponseMessage import InvalidResponseMessage

logger = logging.getLogger("HttpMessageHandler")


class HttpMessageHandler(object):
    def __init__(self, message_handler_builder, cors_origin_hosts=None):
        # type: (MessageHandlerBuilder, Optional[List[str]]) -> None
        self.message_handler_builder = message_handler_builder
        self.cors_origin_hosts = cors_origin_hosts
        if self.cors_origin_hosts is not None:
            self.cors_origin_hosts = {}
            for origin in cors_origin_hosts:
                self.cors_origin_hosts[origin] = True

    def build_singletons(self):
        self.message_handler_builder.build_singletons()

    def handle_request(self, request):
        # type: (HttpRequest) -> HttpResponse
        if self.cors_origin_hosts is not None and request.method == "OPTIONS" and "Origin" in request.headers:
            origin = request.headers["Origin"]
            requested_method = None
            if "Access-Control-Request-Method" in request.headers:
                requested_method = request.headers["Access-Control-Request-Method"]
            origin_host = self._get_origin_host(origin)

            headers = {}
            if (
                    (origin_host not in self.cors_origin_hosts or "*" not in self.cors_origin_hosts)
                    and requested_method is not None
            ):
                headers = {
                    "Access-Control-Allow-Origin": origin,
                    "Access-Control-Allow-Methods": "GET,POST,PUT,DELETE,PATCH"
                }
                if "Access-Control-Request-Headers" in request.headers:
                    headers["Access-Control-Allow-Headers"] = request.headers["Access-Control-Request-Headers"]

            return HttpResponse(200, b"", headers=headers)

        request_msg = HttpRequestMessage(request.url, request.method, request.body, request.query, request.headers)
        try:
            response_msg = self._handle_message(request_msg)
        except Exception as ex:
            logger.exception("Error handling request")
            return HttpResponse(500, "Internal Server Error: {}".format(str(ex)).encode("utf-8"), {})

        try:
            return self._parse_http_response_message(request, response_msg)
        except AttributeError:
            return self._parse_message(response_msg)

    def destroy_singletons(self):
        self.message_handler_builder.destroy_singletons()

    def _handle_message(self, msg):
        message_handler = None
        try:
            message_handler = self.message_handler_builder.build_message_handler(msg, None)
            msg_bus = GetReplyMessageBus()
            message_handler.handle_message(msg_bus, msg)
            return msg_bus.get_reply()
        finally:
            if message_handler is not None:
                self.message_handler_builder.destroy_message_handler(message_handler)

    def _parse_http_response_message(self, request, message):
        http_response = HttpResponse(message.status_code, message.body, message.http_headers)

        if "Access-Control-Allow-Origin" in message.http_headers:
            return http_response

        if self.cors_origin_hosts is None or "Origin" not in request.headers:
            return http_response

        origin = request.headers["Origin"]
        origin_host = self._get_origin_host(origin)
        if "*" not in self.cors_origin_hosts and origin_host not in self.cors_origin_hosts:
            return http_response
        http_response.headers["Access-Control-Allow-Origin"] = origin
        return http_response

    @staticmethod
    def _parse_message(message):
        if message.token != DefaultToken.TK_HTTP_MSG:
            raise InvalidResponseMessage("The response token must be TK_HTTP_MSG")

        try:
            parts = message.data.split(b"\0")
        except AttributeError:
            raise InvalidResponseMessage("Data is None. Must conform to the HttResponseMessagePattern")

        try:
            status_code = int(parts[0])
        except ValueError:
            raise InvalidResponseMessage("Status code could not be converted. "
                                         "Must conform to the HttResponseMessagePattern")
        except IndexError:
            raise InvalidResponseMessage("Data is empty. Must conform to the HttResponseMessagePattern")

        try:
            header_count = int(parts[1])
        except ValueError:
            raise InvalidResponseMessage("Header count could not be converted. "
                                         "Must conform to the HttResponseMessagePattern")
        except IndexError:
            raise InvalidResponseMessage("Header count could not be converted. "
                                         "Must conform to the HttResponseMessagePattern")

        headers = {}
        for i in range(header_count):
            try:
                header_key = parts[2 + 2 * i].decode("utf-8")
                header_value = parts[2 + 2 * i + 1].decode("utf-8")
                headers[header_key] = header_value
            except IndexError:
                raise InvalidResponseMessage("Not enough headers while getting position {}. Header count: {}. "
                                             "Must conform to the HttResponseMessagePattern".format(i, header_count))

        if len(parts) >= 2 + 2 * header_count + 1:
            body = b"\0".join(parts[2 + 2 * header_count:])
        else:
            body = b""

        return HttpResponse(status_code, body, headers)

    def _get_origin_host(self, origin):
        # type: (str) -> str
        index = origin.find("://")
        if index >= 0:
            return origin[index + 3:]
        return origin



from logging import Logger
from xml.etree import cElementTree as eTree

from mbcontextmessagehandler import MbContextMessageBus
from messagebus import (
    Message,
    DataType,
)
from msgbus import (
    TK_PRN_PRINT,
    TK_SYS_NAK,
    MBTimeout,
    MBException,
)
from sysactions import get_used_service
from typing import Optional


class PrinterService(object):
    def __init__(
        self,
        logger,         # type: Logger
        message_bus,    # type: MbContextMessageBus
    ):
        # type: (...) -> None

        self.logger = logger
        self.message_bus = message_bus

    def send_print_data(
        self,           # type: PrinterService
        model,          # type: eTree
        response,       # type: str
    ):
        # type: (...) -> Optional[Message]

        current_printer = get_used_service(
            model=model,
            service_type="printer",
        )

        thirty_seconds = 30 * 1000000
        message = Message(
            token=TK_PRN_PRINT,
            data_type=DataType.param,
            data=response,
            timeout=thirty_seconds,
        )

        print_response = None
        try:
            print_response = self.message_bus.send_message(
                component=current_printer,
                message=message,
            )
            if print_response.token == TK_SYS_NAK:
                self.logger.error("Cannot print data receive NAK")
        except(MBTimeout, MBException) as _:
            self.logger.exception("MB error print data")
        except Exception as _:
            self.logger.exception("Error print data")

        return print_response



from _CancelPaymentProcessor import CancelPaymentProcessor
from _CancelPaymentOrderProcessor import CancelPaymentOrderProcessor
from _ConfirmFinishPaymentProcessor import ConfirmFinishPaymentProcessor
from _FinishPaymentProcessor import FinishPaymentProcessor
from _GetDocumentProcessor import GetDocumentProcessor
from _GetElectronicTypesProcessor import GetElectronicTypesProcessor
from _StartPaymentProcessor import StartPaymentProcessor
from _TestConnectionProcessor import TestConnectionProcessor
from _GetNameProcessor import GetNameProcessor
from _CacheProcessor import CacheProcessor



from threading import Thread, Condition


class BaseThread(Thread):
    def __init__(self):
        # type: () -> None

        super(BaseThread, self).__init__()
        self.running = True
        self.daemon = True
        self.sleep_condition = Condition()

    def stop(self):
        # type: () -> None

        self.running = False
        with self.sleep_condition:
            self.sleep_condition.notify_all()

    def sleep(self, time):
        # type: (int) -> None

        with self.sleep_condition:
            self.sleep_condition.wait(time)

    def wake_up(self):
        # type: () -> None

        with self.sleep_condition:
            self.sleep_condition.notify_all()



import os
import sqlite3
from sqlite3 import Connection
from datetime import datetime

from services import CommonService
from typing import (
    Any,
    Callable,
    Dict,
    List,
    Optional,
    Tuple,
    Union
)

from dbapitransactioncallback import DbApiTransactionCallback
from dbapitemplate import DbApiTemplate
from repository import OrderRepository as BaseOrderRepository


class SqliteOrderRepository(BaseOrderRepository):

    def __init__(
        self,
        common_service,         # type: CommonService
        database_dir,           # type: str
        backup_database_dir,    # type: str
        schema_sql_path,        # type: str
        callback                # type: DbApiTransactionCallback
    ):
        # type: (...) -> None

        self.common_service = common_service
        self.callback = callback
        self.database_dir = database_dir
        self.backup_database_dir = backup_database_dir
        self.schema_sql_path = schema_sql_path

        self.orders_conn = None
        self.multi_template = None

        self.sale_types = None

    def get_backup_database_dir(
        self
    ):
        # type: () -> str

        return self.backup_database_dir

    def get_database_dir(
        self
    ):
        # type: () -> str

        return self.database_dir

    def _get_orders_conn(self):
        if self.orders_conn is None:
            self.orders_conn = self._fill_orders_conn(self.common_service.get_pos_list())

        return self.orders_conn

    def _get_multi_template(self):
        if self.multi_template is None:
            self.multi_template = self._fill_multi_template()

        return self.multi_template

    def _fill_orders_conn(
        self,
        pos_list    # type: List[int]
    ):
        # type: (...) -> Dict[int, Connection]

        orders_conn = {}
        for pos_id in pos_list:
            order_database_path = os.path.join(self.database_dir, "order.db{}".format(pos_id))
            orders_conn[pos_id] = sqlite3.connect(
                database=order_database_path,
                check_same_thread=False
            )
        return orders_conn

    def _fill_multi_template(
        self
    ):
        # type: () -> Dict[int, DbApiTemplate]

        multi_template = {}
        orders_conn = self._get_orders_conn()
        for pos_id in orders_conn:
            multi_template[pos_id] = DbApiTemplate(orders_conn[pos_id])

        return multi_template

    def _update_callback_conn(
        self,
        conn    # type: Connection
    ):
        # type: (...) -> None

        self.callback.update_conn(conn)

    @staticmethod
    def _execute_script(
        connection,     # type: Connection
        query,          # type: str
    ):
        # type: (...) -> None

        connection.executescript(query)

    def _execute_multi_template(
        self,
        pos_id,             # type: str
        query,              # type: str
        params=None,        # type: Tuple
        mapper=None,        # type: Optional[Callable[[tuple], Any]]
        is_script=False     # type: Optional[bool]
    ):
        # type: (...) -> Union[int, list]

        orders_conn = self._get_orders_conn()
        conn = orders_conn[pos_id]
        self._update_callback_conn(conn)

        multi_template = self._get_multi_template()
        if pos_id in multi_template:
            if is_script:
                return self._execute_script(
                    connection=multi_template[pos_id].conn,
                    query=query
                )

            return multi_template[pos_id].execute(
                sql=query,
                params=params if params else (),
                mapper=mapper
            )

    def get_orders(
        self,
        pos_id,         # type: int
        end_date,       # type: str,
        order_states,   # type: List[str]
    ):
        # type: (...) -> List[Tuple[int, str]]

        query = self._get_orders_by_date_query.format("'" + "','".join(order_states) + "'")

        params = (end_date,) if end_date else None

        def mapper(row):
            return row["OrderId"], str(row["BusinessPeriod"])

        return self._execute_multi_template(
            pos_id=pos_id,
            query=query,
            params=params,
            mapper=mapper
        )

    def get_business_period_from_oldest_order(self):
        # type: (...) -> Optional[datetime]
        def mapper(row):
            return datetime.strptime(str(row["BusinessPeriod"]), "%Y%m%d")

        dates_list = []
        query = self._get_created_at_from_oldest_order_query
        for pos_id in self.common_service.get_pos_list():
            dates_list.extend(self._execute_multi_template(
                pos_id=pos_id,
                query=query,
                mapper=mapper
            ))
        dates_list.sort()
        return dates_list[0] if dates_list else None

    def _get_create_tables_query(self):
        # type: () -> str

        try:
            with open(self.schema_sql_path, "r") as f:
                query = f.read()
            return query
        except Exception as _:
            return ""

    def _create_tables_orders(self, db_api):
        # type: (DbApiTemplate) -> None

        query = self._get_table_exists_query
        _, data = db_api.execute(query, ())

        if not data:
            create_table_query = self._get_create_tables_query()
            self._execute_script(
                connection=db_api.conn,
                query=create_table_query
            )

    def fill_order_backup_database(
        self,
        pos_id,         # type: int
        suffix_date,    # type: str
        orders          # type: List[int]
    ):
        # type: (...) -> None

        conn = sqlite3.connect(os.path.join(self.backup_database_dir, "orders_{}.db".format(suffix_date)))
        self._update_callback_conn(conn)
        db_api = DbApiTemplate(conn)

        self._create_tables_orders(db_api)

        order_database_path = os.path.join(self.database_dir, "order.db{}".format(pos_id))
        query = self._backup_order_database_query.format(order_database_path, ','.join([str(o) for o in orders]))

        self._execute_script(db_api.conn, query)

    def delete_orders(
        self,
        pos_id,     # int
        orders      # type: List[int]
    ):
        # type: (...) -> None

        query = self._get_delete_orders_query.format(','.join([str(o) for o in orders]))
        self._execute_multi_template(
            pos_id=pos_id,
            query=query,
            is_script=True
        )

    _get_table_exists_query = """
        SELECT count(name) FROM sqlite_master WHERE type='table' AND name='Orders'
    """

    _get_orders_by_date_query = """
        SELECT OrderId, BusinessPeriod FROM Orders
        WHERE BusinessPeriod < ?
        AND StateId IN (
            SELECT StateId 
            FROM OrderState 
            WHERE StateDescr IN ({})
        )
        ORDER BY OrderId LIMIT 1000
    """

    _backup_order_database_query = """
        ATTACH DATABASE '{0}' AS o; 
        INSERT OR REPLACE INTO Orders SELECT * FROM o.Orders WHERE OrderId IN ({1});
        INSERT OR REPLACE INTO OrderCustomProperties SELECT * FROM o.OrderCustomProperties WHERE OrderId IN ({1});
        INSERT OR REPLACE INTO OrderItemCustomProperties SELECT * FROM o.OrderItemCustomProperties WHERE OrderId IN ({1});
        INSERT OR REPLACE INTO OrderStateHistory SELECT * FROM o.OrderStateHistory WHERE OrderId IN ({1});
        INSERT OR REPLACE INTO OrderItem SELECT * FROM o.OrderItem WHERE OrderId IN ({1});
        INSERT OR REPLACE INTO OrderTax SELECT * FROM o.OrderTax WHERE OrderId IN ({1});
        INSERT OR REPLACE INTO OrderDiscount SELECT * FROM o.OrderDiscount WHERE OrderId IN ({1});
        INSERT OR REPLACE INTO OrderVoidHistory SELECT * FROM o.OrderVoidHistory WHERE OrderId IN ({1});
        INSERT OR REPLACE INTO OrderTender SELECT * FROM o.OrderTender WHERE OrderId IN ({1});
        INSERT OR REPLACE INTO OrderComment SELECT * FROM o.OrderComment WHERE OrderId IN ({1});
        INSERT OR REPLACE INTO OrderProduct SELECT * FROM o.OrderProduct WHERE OrderId IN ({1});
        INSERT OR REPLACE INTO OrderPrice SELECT * FROM o.OrderPrice WHERE OrderId IN ({1});
        DETACH o;
    """

    _get_delete_orders_query = """
        DELETE FROM OrderCustomProperties WHERE OrderId IN ({0});
        DELETE FROM OrderItemCustomProperties WHERE OrderId IN ({0});
        DELETE FROM OrderStateHistory WHERE OrderId IN ({0});
        DELETE FROM OrderItem WHERE OrderId IN ({0});
        DELETE FROM OrderTax WHERE OrderId IN ({0});
        DELETE FROM OrderDiscount WHERE OrderId IN ({0});
        DELETE FROM OrderVoidHistory WHERE OrderId IN ({0});
        DELETE FROM OrderTender WHERE OrderId IN ({0});
        DELETE FROM OrderComment WHERE OrderId IN ({0});
        DELETE FROM OrderProduct WHERE OrderId IN ({0});
        DELETE FROM Orders WHERE OrderId IN ({0});
    """

    _get_created_at_from_oldest_order_query = """
            SELECT BusinessPeriod FROM Orders ORDER BY CreatedAt LIMIT 1; 
        """



# -*- coding: utf-8 -*-
import json
import os

from datetime import datetime

import cfgtools
from old_helper import BaseRepository

from utilfunctions import (
    get_tender_info,
    is_electronic_payment,
)

from actions.models import (
    GLAccount,
    TransferType,
    CloseOperator,
)


class AccountRepository(BaseRepository):
    def __init__(self, mb_context):
        super(AccountRepository, self).__init__(mb_context)
        config = cfgtools.read(os.environ["LOADERCFG"])
        self.considers_tef = config.find_value(
            "Customizations.ConsidersTEFOnBordereau",
            "false",
        ).lower() == "true"
        self.bordereau_enabled = config.find_value(
            "Customizations.BordereauEnabled",
            "false",
        ).lower() == "true"

    def insert_close_operator_info(self, operator):
        # type: (CloseOperator) -> float

        from table_actions import considers_tef_on_bordereau

        def inner_func(conn):
            total_bordereau_difference = 0
            initial_fund = self.get_initial_amount(operator.pos_id, operator.session)
            total_cash_in = self.get_cash_in_total_amount(operator.pos_id, operator.session)
            total_cash_out = self.get_cash_out_total_amount(operator.pos_id, operator.session)
            final_cash = initial_fund + total_cash_in - total_cash_out

            if self.bordereau_enabled and considers_tef_on_bordereau:
                _link_tef_tenders(operator)

            _insert_unlinked_tef_tenders(operator)

            query = "DELETE FROM Transfer WHERE SessionId = '{}';".format(operator.session)

            for value in operator.values:
                tender_id = int(value["tenderId"])
                sold_amount = sum(p["amount"] for p in operator.payments.get(tender_id, []))
                sold_amount += value.get("additionalSkim", 0.0)

                declared_amount = value["amount"]
                if declared_amount is not None:
                    tender_difference = _get_total_difference(
                        tender_id,
                        declared_amount,
                        sold_amount,
                        final_cash,
                    )
                    total_bordereau_difference += abs(tender_difference)
                else:
                    declared_amount = sold_amount

                envelope = _get_envelope(tender_id, operator)
                gl_account = GLAccount(manager=operator.manager_id, envelope=envelope)
                gl_account_json = json.dumps(gl_account, default=lambda o: o.__dict__, sort_keys=True)

                query += """
                        INSERT INTO Transfer 
                        VALUES ('{}', {}, '{}', {}, '{}', {}, {}, '{}', '{}');
                    """.format(
                        operator.period,
                        operator.pos_id,
                        operator.session,
                        TransferType.DECLARED_AMOUNT,
                        "DECLARED_AMOUNT",
                        declared_amount,
                        tender_id,
                        datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"),
                        gl_account_json
                    )

                query += """
                        INSERT INTO Transfer 
                        VALUES ('{}', {}, '{}', {}, '{}', {}, {}, '{}', '{}');
                    """.format(
                        operator.period,
                        operator.pos_id,
                        operator.session,
                        TransferType.TRANSFER_SKIM,
                        "TRANSFER_SKIM",
                        sold_amount,
                        tender_id,
                        datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"),
                        ""
                    )

            conn.query(query)

            return total_bordereau_difference

        return self.execute_with_transaction(inner_func, db_name=str(operator.pos_id))

    def update_justification(self, pos_id, session, justification):
        def inner_func(conn):
            conn.query("""UPDATE Transfer
                          SET GLAccount = REPLACE(GLAccount, '"justification": ""', '"justification": {}')
                          WHERE SessionId = '{}' AND GLAccount IS NOT NULL AND GLAccount != '' AND Type IN (5,6)"""
                       .format(self._fix_str_to_json_format(justification), session))

        self.execute_with_transaction(inner_func, db_name=str(pos_id))

    @staticmethod
    def _fix_str_to_json_format(
        string,  # type: str
    ):
        # type: (...) -> str

        return json.dumps(string.replace("'", "\""))

    def get_closing_operator_sessions(self, pos_id, user, start_date, end_date):
        def inner_func(conn):
            sessions = []
            query = """SELECT SessionId
                       FROM Transfer
                       WHERE SessionId LIKE '%user={},%' and Period >= '{}' and Period <= '{}'
                       GROUP BY SessionId
                       ORDER BY Timestamp""".format(user, start_date, end_date)

            cursor = conn.select(query)
            if cursor.rows() > 0:
                for row in cursor:
                    sessions.append(row.get_entry("SessionId"))

            return sessions

        return self.execute_with_connection(inner_func, db_name=str(pos_id))

    def get_initial_amount(self, pos_id, session_id):
        return self.get_transfer_total_amount(pos_id, session_id, TransferType.INITIAL_AMOUNT)

    def get_cash_in_total_amount(self, pos_id, session_id):
        return self.get_transfer_total_amount(pos_id, session_id, TransferType.TRANSFER_CASH_IN)

    def get_cash_out_total_amount(self, pos_id, session_id):
        return self.get_transfer_total_amount(pos_id, session_id, TransferType.TRANSFER_CASH_OUT)

    def get_transfer_total_amount(self, pos_id, session_id, transfer_type):
        def inner_func(conn):
            query = """
                        SELECT amount FROM transfer
                        WHERE Type = '{}' AND SessionId = '{}'
                    """.format(transfer_type, session_id)
            cursor = conn.select(query)
            if cursor.rows() > 0:
                total_cash_in = 0
                for row in cursor:
                    total_cash_in += float(row.get_entry(0) or 0.0)

                return total_cash_in

            return 0

        return self.execute_with_connection(inner_func, db_name=str(pos_id))


def _get_envelope(tender_id, operator):
    if tender_id == 0:
        return operator.envelope

    return ""


def _get_total_difference(
    tender_id,      # type: int
    amount,         # type: float
    sold_amount,    # type: float
    final_cash,     # type: float
):
    # type: (...) -> float

    if tender_id == 0:
        return (amount - sold_amount) - final_cash

    return amount - sold_amount


def _link_tef_tenders(
    operator,  # type: CloseOperator
):
    # type: (...) -> None

    for value in operator.values:
        if not value.get("linkWithTef"):
            continue

        custom_attr = value.get("customAttr", {})
        sitef_code = custom_attr.get("sitefCode")
        if not sitef_code:
            continue

        sitef_codes = str(sitef_code).split(";")
        for payment_tender_id, payment_list in operator.payments.items():
            payment_tender = get_tender_info(payment_tender_id)
            if not is_electronic_payment(payment_tender):
                continue

            for payment in payment_list:
                payment_sitef_code = str(payment.get("cardBrand", ""))
                if payment_sitef_code not in sitef_codes:
                    continue

                if "additionalSkim" not in value:
                    value["additionalSkim"] = 0

                value["additionalSkim"] += payment["amount"]
                payment["amount"] = 0.0
                payment["linked"] = True


def _insert_unlinked_tef_tenders(
    operator,  # type: CloseOperator
):
    # type: (...) -> None

    values_tender_ids = [value["tenderId"] for value in operator.values]
    for tender_id, payment_list in operator.payments.items():
        if tender_id in values_tender_ids:
            continue

        tender = get_tender_info(tender_id)
        if not is_electronic_payment(tender):
            continue

        operator.values.append({
            "amount": None,
            "tenderId": tender_id,
            "linkWithTef": False,
            "customAttr": {},
        })



from ._ErrorType import ErrorType
from ._NfeTender import NfeTender
from ._PosTenderToNfeTender import PosTenderToNfeTender
from ._NfeIntermediary import NfeIntermediary
from ._NfePresence import NfePresence
from ._NfeCstatCode import NfceCstatCode

from softwaredeveloper import SoftwareDeveloper

__all__ = [
    "ErrorType",
    "SoftwareDeveloper",
    "PosTenderToNfeTender",
    "NfeTender",
    "NfePresence",
    "NfeIntermediary",
    "NfceCstatCode",
]



from ._Benefit import (
    add_to_dict_value,
    benefit_is_add_item_type_on_total,
    get_applied_benefits_by_key,
    get_benefit_key,
    get_benefit_key_from_persisted_benefits,
    get_benefit_to_apply_from_persisted_benefits,
    get_dict_data_remove,
    get_error_message,
    get_item_values_line_ids,
    get_line_id_from_sale_line,
    get_list_data_remove,
    get_new_benefit,
    get_remaining_amount_discount,
    have_action_by_items_or_groups,
    is_invalid_sale_line_or_remaining_quantity,
    is_order_taker,
    remove_dict_properties_benefits,
    remove_list_properties_benefits,
    round_value,
    split_benefit_key,
    subtract_quantity,
    sum_discounts,
)
from ._CompositionType import CompositionType
from ._DictionaryOperations import DictionaryOperations
from ._Range import get_range_discount_ids
from ._ServicePeriod import get_service_period
from ._User import get_user_data
from ._Wait import wait




# -*- coding: utf-8 -*-
import base64
import gzip
import json
import threading
from cStringIO import StringIO
from xml.etree import cElementTree as eTree

import requests
from syserrors import SE_SUCCESS, SE_BADXML, SE_BADRESPONSE
from typing import Optional

from application.configurator import Configurator
from application.eventsender import EventSender
from application.interactor import (
    OrderPictureInteractor,
    OperatorLogoutInteractor,
    ProductionOrderInteractor,
    TablePictureInteractor,
    StoreStatusInteractor,
)
from application.model import EventData, AuditEvent, Command
from ._AuditEventConsumer import AuditEventConsumer


class AuditEventConsumerEventHandler(AuditEventConsumer):

    def __init__(self, configurations):
        # type: (Configurator) -> None

        self.configs = configurations
        self.repository = None
        self.event_sender = EventSender(self.configs)

        self.operator_logout_interactor = None
        self.table_picture_interactor = None
        self.production_order_interactor = None
        self.store_status_interactor = None

        super(self.__class__, self).__init__(self.configs, self.process)

        with self.lock:
            self._create_commands_table()

        t = threading.Thread(target=self._status_sender)
        t.daemon = True
        t.start()

    def set_repository(self, repository):
        self.repository = repository

    def build_interactors(self):
        # type: () -> None

        self.operator_logout_interactor = OperatorLogoutInteractor(self.configs, self.repository)
        self.table_picture_interactor = TablePictureInteractor(self.configs)
        self.production_order_interactor = ProductionOrderInteractor(self.configs)
        self.store_status_interactor = StoreStatusInteractor(self.configs)

    def process(self, evt_list):
        for evt in evt_list:
            try:
                data, evt_type = evt[0], evt[1]
                self.logger.info("New event received: [{}]".format(evt_type))
                event_data = self._get_event_data(evt_type, data)
            except Exception as _:
                message = "Auditlog event parser error. Exiting to force resynchronization. Event: {}".format(evt)
                self.logger.exception(message)
                return SE_BADXML

            try:
                events_to_generate_order_picture = [
                    AuditEvent.PAID,
                    AuditEvent.VOID_ORDER,
                    AuditEvent.ORDER_REPROCESS,
                    AuditEvent.CLEAR_TENDERS
                ]
                if evt_type in events_to_generate_order_picture:
                    order_picture_interactor = OrderPictureInteractor(self.configs, self.repository)
                    payload = order_picture_interactor.build(event_data)
                    result = self.event_sender.send_request(event_data, payload)
                    if result != SE_SUCCESS:
                        return result

                elif evt_type in [AuditEvent.TABLE_CLOSED, ]:
                    payload = self.table_picture_interactor.build(event_data)
                    result = self.event_sender.send_request(event_data, payload, True)
                    if result != SE_SUCCESS:
                        return result

                elif evt_type == AuditEvent.OPERATOR_LOGOUT:
                    payload = self.operator_logout_interactor.build(event_data)
                    if not payload:
                        continue
                    result = self.event_sender.send_request(event_data, payload)
                    if result != SE_SUCCESS:
                        return result

                elif evt_type == AuditEvent.PRODUCTION_ORDER:
                    payload = self.production_order_interactor.build(event_data)
                    result = self.event_sender.send_request(event_data, payload)
                    if result != SE_SUCCESS:
                        return result

                elif evt_type in [AuditEvent.CLOSE_STORE_DAY, AuditEvent.OPEN_STORE_DAY]:
                    payload = self.store_status_interactor.build(event_data)
                    result = self.event_sender.send_request(event_data, payload)
                    if result != SE_SUCCESS:
                        return result

            except Exception as _:
                self.logger.exception("[{}] Unable to process event".format(evt_type))
                return SE_BADRESPONSE

        return SE_SUCCESS

    def _status_sender(self):
        ctrl_conn = self._create_db_conn()

        while not self.terminating:
            wait_time = 15
            try:
                command = None
                if self.repository is not None:
                    order_picture = self.repository.get_last_order_picture()
                    if order_picture is not None:
                        last_command_id = self._get_last_command_id(ctrl_conn)
                        command = self._send_status(order_picture, last_command_id)
                    wait_time = 600

                if command and command.type == "ReprocessSales":
                    self.logger.info("Received ReprocessSales Command")
                    self._update_command_id(command.id, ctrl_conn)
                    self.force_new_period(command.data, ctrl_conn)

            except WindowsError as _:
                self.renew_service()
                self.logger.exception("On WindowsError")
            except Exception as _:
                wait_time = 60
                self.logger.exception("ERROR on PUMP Status Thread")

            finally:
                with self.condition:
                    self.condition.wait(wait_time)

    def _create_commands_table(self, ctrl_conn=None):
        if not ctrl_conn:
            ctrl_conn = self.ctrl_conn

        ctrl_conn.execute("""
                CREATE TABLE IF NOT EXISTS CommandsProcessed (
                    CommandId   INTEGER NOT NULL,
                    PRIMARY KEY (CommandId)
                );
            """)

    def _update_command_id(self, command_id, ctrl_conn=None):
        if not ctrl_conn:
            ctrl_conn = self.ctrl_conn

        query = "INSERT OR IGNORE INTO CommandsProcessed(CommandId) VALUES(:command_id);"
        ctrl_conn.execute(query, {"command_id": command_id})

    def _get_last_command_id(self, ctrl_conn=None):
        if not ctrl_conn:
            ctrl_conn = self.ctrl_conn

        c = ctrl_conn.cursor()
        r = c.execute("""SELECT MAX(CommandId) FROM CommandsProcessed;""").fetchone()
        return r[0]

    def _send_status(self, order_picture, last_command_id):
        # type: (str, str) -> Optional[Command]

        xml_picture = eTree.fromstring(order_picture)
        order = xml_picture.find(".//Order")
        if order is None or order.get("orderId") is None:
            raise Exception()

        message = {
            "lastOrderId": order.get("orderId"),
            "lastOrderDatetime": order.get("createdAtGMT") + "Z",
            "lastCommandIdProcessed": last_command_id
        }

        payload = json.dumps(message)
        try:
            payload = payload.encode("utf-8")
            self.logger.info("Sending Pump Status to server. Payload:\n{}".format(payload))

            headers = {
                "Accept": "application/json",
                "Content-type": "application/json",
                "x-api-key": self.configs.api_key
            }

            post_url = self.configs.server_url + self.configs.pump_status_url + "/{}".format(self.configs.store_id)
            response = requests.post(post_url, headers=headers, data=payload, timeout=30, verify=False)  # NOSONAR

            if response.status_code != 200:
                log_message = "Error sending Pump Status server. Status: {} - {}"
                self.logger.info(log_message.format(response.status_code, response.content))
                return None

            self.logger.info("Success sending Pump Status to server")

            if response.content == "":
                return None
            return Command(response.json())

        except Exception as _:
            message = "Communication error sendingPump Status to server"
            self.logger.exception(message)
            return None

    @staticmethod
    def _get_event_data(evt_type, xml):
        # type: (str, str) -> EventData

        event = eTree.XML(xml)
        audit_log_evt = event.find("AuditLog")
        evt_timestamp = audit_log_evt.get("evtTime") or ""
        evt_data = audit_log_evt.text.encode("UTF-8")
        data = eTree.fromstring(gzip.GzipFile(fileobj=StringIO(base64.b64decode(evt_data))).read())
        session_id = data.get("sessionid")

        return EventData(data=data, event_type=evt_type, timestamp=evt_timestamp, session_id=session_id)



import locale
from datetime import datetime

from pos_model import TenderType
from typing import Dict, List, Optional, Tuple

from .model import (
    Order,
    TotalTender,
)
from repository import (
    OrderRepository,
    FiscalRepository,
    TenderRepository,
)
from utils.report_formatters import translate_pos_label


class BrandReport(object):
    TYPE_REAL_DATE = "RealDate"
    TYPE_BUSINESS_PERIOD = "BusinessPeriod"
    TYPE_SESSION_ID = "SessionId"

    def __init__(self, order_repository, fiscal_repository, tender_repository, store_id):
        # type: (OrderRepository, FiscalRepository, TenderRepository, unicode) -> None
        self.order_repository = order_repository
        self.fiscal_repository = fiscal_repository
        self.tender_repository = tender_repository
        self.store_id = store_id

        self.tender_names_dict = {}  # type: Dict[int, unicode]
        for tender_tuple in self.tender_repository.get_tender_names():
            self.tender_names_dict[tender_tuple[0]] = tender_tuple[1]

    def generate_brand_report_by_real_date(self, pos_id, report_pos, initial_date, end_date, operator_id):
        # type: (int, Optional[unicode], datetime, datetime, Optional[unicode]) -> str
        return self._generate_brand_report(self.TYPE_REAL_DATE, pos_id, report_pos, initial_date, end_date, operator_id, None)

    def generate_brand_report_by_business_period(self, pos_id, report_pos, initial_date, end_date, operator_id):
        # type: (int, Optional[unicode], datetime, datetime, unicode) -> str
        return self._generate_brand_report(self.TYPE_BUSINESS_PERIOD, pos_id, report_pos, initial_date, end_date, operator_id, None)

    def generate_brand_report_by_session_id(self, pos_id, session_id):
        # type: (int, unicode) -> str
        return self._generate_brand_report(self.TYPE_SESSION_ID, pos_id, None, None, None, None, session_id)

    def _generate_brand_report(self, report_type, pos_id, report_pos, initial_date, end_date, operator_id, session_id):
        # type: (unicode, int, Optional[unicode], Optional[datetime], Optional[datetime], Optional[unicode], Optional[unicode]) -> str

        if report_type == self.TYPE_REAL_DATE:
            paid_orders = self.order_repository.get_paid_orders_by_real_date(initial_date, end_date, operator_id, report_pos)

        elif report_type == self.TYPE_BUSINESS_PERIOD:
            paid_orders = self.order_repository.get_paid_orders_by_business_period(initial_date, end_date, operator_id, report_pos)

        elif report_type == self.TYPE_SESSION_ID:
            paid_orders = self.order_repository.get_paid_orders_by_session_id(session_id)
            start_index = session_id.find("user=")
            end_index = session_id.find(",", start_index)
            operator_id = session_id[start_index + 5:end_index]
            start_index = session_id.find("period=")
            initial_date = end_date = datetime.strptime(session_id[start_index + 7:], "%Y%m%d")

        else:
            raise NotImplementedError()

        count_pos_error = filter(lambda x: x == "error", paid_orders)
        count_pos_error = len(count_pos_error)

        paid_orders = filter(lambda x: x != "error", paid_orders)

        card_brands = self.fiscal_repository.get_card_brands(paid_orders)

        header = self._build_header(report_type, pos_id, report_pos, initial_date, end_date, operator_id, count_pos_error)
        body = self._build_body(paid_orders, card_brands)

        report = header + body
        report_bytes = report.encode("utf-8")

        return report_bytes

    def _build_header(self, report_type, pos_id, report_pos, initial_date, end_date, operator_id, count_pos_error=0):
        # type: (unicode, int, Optional[unicode], datetime, datetime, unicode) -> unicode
        """
        ======================================
          Relatorio de Pagamentos por Bandeira
                        (Loja)
          Loja..........: 99999
          Data/hora.....: 01/08/2017 09:59:26
          Dia Util......: 31/07/2017
          ID Operador ..: Todos (POS # 03)
          POS incluido..: Todos
        ======================================"""
        operator = "Todos"
        if operator_id is not None:
            operator = operator_id

        pos_list = "Todos" if report_pos is None else report_pos.zfill(2)
        pos = translate_pos_label(pos_id=pos_id)
        if report_type == self.TYPE_REAL_DATE:
            report_type_text = "Data Fiscal"
        elif report_type == self.TYPE_BUSINESS_PERIOD:
            report_type_text = "Dia Negocio"
        else:
            report_type_text = "Surpresa"
            pos_list = pos_id

        header = u"======================================\n"
        header += u" Relatorio de Pagamentos por Bandeira  \n"
        header += u"({})".format(report_type_text).center(38) + "\n"
        header += u" Loja.........: " + self.store_id + "\n"
        header += u" Data/hora....: " + datetime.now().strftime("%d/%m/%Y %H:%M:%S") + "\n"
        header += u" {0:.<13}: {1} - {2}\n".format(report_type_text, initial_date.strftime("%d/%m/%y"), end_date.strftime("%d/%m/%y"))
        header += u" ID Operador..: " + u"{0} ({1} # {2})\n".format(operator, pos, str(pos_id).zfill(2))
        header += u" {0} incluido.: ".format(pos) + u"{0}\n".format(pos_list)
        if count_pos_error > 0:
            header += u" {0} excluidos: ".format(pos) + u"{0}\n".format(count_pos_error)
        header += u"======================================\n"

        return header

    def _build_body(self, paid_orders, card_brands):
        # type: (List[Order], List[Tuple[unicode, int, float, int]]) -> unicode

        """
        Descricao          Qtd    Total
          DINHEIRO.......: [XXXX] R$ 00.000,00
          CARTAO CREDITO.: [XXXX] R$ 00.000,00
          CARTAO DEBITO..: [XXXX] R$ 00.000,00
          SpoonRocket....: [XXXX] R$ 00.000,00
        ======================================
        :param paid_orders: the list of paid orders
        :param voided_orders: the list of voided orders
        :param transfers: the list of all transfers
        :return: the report bytes encoded with UTF-8
        """

        total_tender = ""
        total_tender_by_type_dict = {}  # type: Dict[int, TotalTender]
        for order in paid_orders:
            for tender in order.tenders:
                if tender.type in total_tender_by_type_dict:
                    total_tender = total_tender_by_type_dict[tender.type]
                else:
                    total_tender = TotalTender()
                    total_tender.tender_type = tender.type
                    total_tender.quantity = 0
                    if total_tender.tender_type in self.tender_names_dict:
                        total_tender.tender_name = self.tender_names_dict[total_tender.tender_type]
                    else:
                        total_tender.tender_name = "OUTROS"
                    total_tender_by_type_dict[tender.type] = total_tender

                total_tender.quantity += 1
                total_tender.value += tender.value

        total_tender_by_type_list = []  # type: List[TotalTender]
        for tender_type_id in total_tender_by_type_dict.keys():
            tender_type = total_tender_by_type_dict[tender_type_id]
            tender_type.name = total_tender.tender_name

            total_tender_by_type_list.append(tender_type)

        total_tender_by_type_list = sorted(total_tender_by_type_list, key=lambda x: x.tender_type)

        try:
            locale.setlocale(locale.LC_ALL, "portuguese_brazil")
        except Exception as _:
            locale.setlocale(locale.LC_ALL, "pt_BR.UTF-8")

        body = u"Descricao         Qtd    Total\n"
        if card_brands is None:
            for total_tender in total_tender_by_type_list:
                body += u"{0:.<15}: [{1:>4}] R${2:>10}\n".format(total_tender.tender_name,
                                                                   total_tender.quantity,
                                                                   locale.format("%.2f", total_tender.value, True))
        else:
            for total_tender in total_tender_by_type_list:
                if total_tender.tender_type not in (TenderType.credit, TenderType.debit, TenderType.external_card):
                    body += u"{0:.<16}: [{1:>4}] R${2:>10}\n".format(total_tender.tender_name,
                                                                      total_tender.quantity,
                                                                      locale.format("%.2f", total_tender.value, True))
                else:
                    body += u"{0:.<16}:\n".format(total_tender.tender_name)
                    for brand in card_brands:
                        if total_tender.tender_type == brand[1]:
                            body += u"  {0:.<15}: [{1:>4}] R${2:>10}\n".format(brand[0][:15],
                                                                               brand[3],
                                                                               locale.format("%.2f", brand[2], True))
        body += u"======================================\n"

        return body



from typing import List

from report import Presenter  # noqa
from basereport import SimpleReport

from .builders import TableReportHeader, TableReportBody, ReportBuilder
from .dto import TableReportDto  # noqa


class TableReportPresenter(Presenter):
    def __init__(
        self,
        print_by,                   # type: str
        hide_unpriced_items,        # type: bool
        pre_account_with_discount,  # type: bool
        pos_id,                     # type: str
        filtered_seat               # type: str
    ):
        # type: (...) -> None

        self.print_by = print_by
        self.hide_unpriced_items = hide_unpriced_items
        self.pre_account_with_discount = pre_account_with_discount
        self.pos_id = pos_id
        self.filtered_seat = filtered_seat

    def present(self, dto):
        # type: (TableReportDto) -> SimpleReport

        pos_id = self.pos_id
        report_builders = [
            TableReportHeader(dto.header, self.pre_account_with_discount),
            TableReportBody(
                table_report_body_dto=dto.body,
                print_by=self.print_by,
                hide_unpriced_items=self.hide_unpriced_items,
                pre_account_with_discount=self.pre_account_with_discount,
                pos_id=pos_id,
                filtered_seat=self.filtered_seat
            )
        ]  # type: List[ReportBuilder]

        report = []
        for builder in report_builders:
            report.extend(builder.generate())

        return SimpleReport(report, 38)



# Copyright Jonathan Hartley 2013. BSD 3-Clause license, see LICENSE file.
from io import StringIO, TextIOWrapper
from unittest import TestCase, main
try:
    from contextlib import ExitStack
except ImportError:
    # python 2
    from contextlib2 import ExitStack

try:
    from unittest.mock import MagicMock, Mock, patch
except ImportError:
    from mock import MagicMock, Mock, patch

from ..ansitowin32 import AnsiToWin32, StreamWrapper
from ..win32 import ENABLE_VIRTUAL_TERMINAL_PROCESSING
from .utils import osname


class StreamWrapperTest(TestCase):

    def testIsAProxy(self):
        mockStream = Mock()
        wrapper = StreamWrapper(mockStream, None)
        self.assertTrue( wrapper.random_attr is mockStream.random_attr )

    def testDelegatesWrite(self):
        mockStream = Mock()
        mockConverter = Mock()
        wrapper = StreamWrapper(mockStream, mockConverter)
        wrapper.write('hello')
        self.assertTrue(mockConverter.write.call_args, (('hello',), {}))

    def testDelegatesContext(self):
        mockConverter = Mock()
        s = StringIO()
        with StreamWrapper(s, mockConverter) as fp:
            fp.write(u'hello')
        self.assertTrue(s.closed)

    def testProxyNoContextManager(self):
        mockStream = MagicMock()
        mockStream.__enter__.side_effect = AttributeError()
        mockConverter = Mock()
        with self.assertRaises(AttributeError) as excinfo:
            with StreamWrapper(mockStream, mockConverter) as wrapper:
                wrapper.write('hello')

    def test_closed_shouldnt_raise_on_closed_stream(self):
        stream = StringIO()
        stream.close()
        wrapper = StreamWrapper(stream, None)
        self.assertEqual(wrapper.closed, True)

    def test_closed_shouldnt_raise_on_detached_stream(self):
        stream = TextIOWrapper(StringIO())
        stream.detach()
        wrapper = StreamWrapper(stream, None)
        self.assertEqual(wrapper.closed, True)

class AnsiToWin32Test(TestCase):

    def testInit(self):
        mockStdout = Mock()
        auto = Mock()
        stream = AnsiToWin32(mockStdout, autoreset=auto)
        self.assertEqual(stream.wrapped, mockStdout)
        self.assertEqual(stream.autoreset, auto)

    @patch('colorama.ansitowin32.winterm', None)
    @patch('colorama.ansitowin32.winapi_test', lambda *_: True)
    def testStripIsTrueOnWindows(self):
        with osname('nt'):
            mockStdout = Mock()
            stream = AnsiToWin32(mockStdout)
            self.assertTrue(stream.strip)

    def testStripIsFalseOffWindows(self):
        with osname('posix'):
            mockStdout = Mock(closed=False)
            stream = AnsiToWin32(mockStdout)
            self.assertFalse(stream.strip)

    def testWriteStripsAnsi(self):
        mockStdout = Mock()
        stream = AnsiToWin32(mockStdout)
        stream.wrapped = Mock()
        stream.write_and_convert = Mock()
        stream.strip = True

        stream.write('abc')

        self.assertFalse(stream.wrapped.write.called)
        self.assertEqual(stream.write_and_convert.call_args, (('abc',), {}))

    def testWriteDoesNotStripAnsi(self):
        mockStdout = Mock()
        stream = AnsiToWin32(mockStdout)
        stream.wrapped = Mock()
        stream.write_and_convert = Mock()
        stream.strip = False
        stream.convert = False

        stream.write('abc')

        self.assertFalse(stream.write_and_convert.called)
        self.assertEqual(stream.wrapped.write.call_args, (('abc',), {}))

    def assert_autoresets(self, convert, autoreset=True):
        stream = AnsiToWin32(Mock())
        stream.convert = convert
        stream.reset_all = Mock()
        stream.autoreset = autoreset
        stream.winterm = Mock()

        stream.write('abc')

        self.assertEqual(stream.reset_all.called, autoreset)

    def testWriteAutoresets(self):
        self.assert_autoresets(convert=True)
        self.assert_autoresets(convert=False)
        self.assert_autoresets(convert=True, autoreset=False)
        self.assert_autoresets(convert=False, autoreset=False)

    def testWriteAndConvertWritesPlainText(self):
        stream = AnsiToWin32(Mock())
        stream.write_and_convert( 'abc' )
        self.assertEqual( stream.wrapped.write.call_args, (('abc',), {}) )

    def testWriteAndConvertStripsAllValidAnsi(self):
        stream = AnsiToWin32(Mock())
        stream.call_win32 = Mock()
        data = [
            'abc\033[mdef',
            'abc\033[0mdef',
            'abc\033[2mdef',
            'abc\033[02mdef',
            'abc\033[002mdef',
            'abc\033[40mdef',
            'abc\033[040mdef',
            'abc\033[0;1mdef',
            'abc\033[40;50mdef',
            'abc\033[50;30;40mdef',
            'abc\033[Adef',
            'abc\033[0Gdef',
            'abc\033[1;20;128Hdef',
        ]
        for datum in data:
            stream.wrapped.write.reset_mock()
            stream.write_and_convert( datum )
            self.assertEqual(
               [args[0] for args in stream.wrapped.write.call_args_list],
               [ ('abc',), ('def',) ]
            )

    def testWriteAndConvertSkipsEmptySnippets(self):
        stream = AnsiToWin32(Mock())
        stream.call_win32 = Mock()
        stream.write_and_convert( '\033[40m\033[41m' )
        self.assertFalse( stream.wrapped.write.called )

    def testWriteAndConvertCallsWin32WithParamsAndCommand(self):
        stream = AnsiToWin32(Mock())
        stream.convert = True
        stream.call_win32 = Mock()
        stream.extract_params = Mock(return_value='params')
        data = {
            'abc\033[adef':         ('a', 'params'),
            'abc\033[;;bdef':       ('b', 'params'),
            'abc\033[0cdef':        ('c', 'params'),
            'abc\033[;;0;;Gdef':    ('G', 'params'),
            'abc\033[1;20;128Hdef': ('H', 'params'),
        }
        for datum, expected in data.items():
            stream.call_win32.reset_mock()
            stream.write_and_convert( datum )
            self.assertEqual( stream.call_win32.call_args[0], expected )

    def test_reset_all_shouldnt_raise_on_closed_orig_stdout(self):
        stream = StringIO()
        converter = AnsiToWin32(stream)
        stream.close()

        converter.reset_all()

    def test_wrap_shouldnt_raise_on_closed_orig_stdout(self):
        stream = StringIO()
        stream.close()
        with \
            patch("colorama.ansitowin32.os.name", "nt"), \
            patch("colorama.ansitowin32.winapi_test", lambda: True):
                converter = AnsiToWin32(stream)
        self.assertTrue(converter.strip)
        self.assertFalse(converter.convert)

    def test_wrap_shouldnt_raise_on_missing_closed_attr(self):
        with \
            patch("colorama.ansitowin32.os.name", "nt"), \
            patch("colorama.ansitowin32.winapi_test", lambda: True):
                converter = AnsiToWin32(object())
        self.assertTrue(converter.strip)
        self.assertFalse(converter.convert)

    def testExtractParams(self):
        stream = AnsiToWin32(Mock())
        data = {
            '':               (0,),
            ';;':             (0,),
            '2':              (2,),
            ';;002;;':        (2,),
            '0;1':            (0, 1),
            ';;003;;456;;':   (3, 456),
            '11;22;33;44;55': (11, 22, 33, 44, 55),
        }
        for datum, expected in data.items():
            self.assertEqual(stream.extract_params('m', datum), expected)

    def testCallWin32UsesLookup(self):
        listener = Mock()
        stream = AnsiToWin32(listener)
        stream.win32_calls = {
            1: (lambda *_, **__: listener(11),),
            2: (lambda *_, **__: listener(22),),
            3: (lambda *_, **__: listener(33),),
        }
        stream.call_win32('m', (3, 1, 99, 2))
        self.assertEqual(
            [a[0][0] for a in listener.call_args_list],
            [33, 11, 22] )

    def test_osc_codes(self):
        mockStdout = Mock()
        stream = AnsiToWin32(mockStdout, convert=True)
        with patch('colorama.ansitowin32.winterm') as winterm:
            data = [
                '\033]0\x07',                      # missing arguments
                '\033]0;foo\x08',                  # wrong OSC command
                '\033]0;colorama_test_title\x07',  # should work
                '\033]1;colorama_test_title\x07',  # wrong set command
                '\033]2;colorama_test_title\x07',  # should work
                '\033]' + ';' * 64 + '\x08',       # see issue #247
            ]
            for code in data:
                stream.write(code)
            self.assertEqual(winterm.set_title.call_count, 2)

    def test_native_windows_ansi(self):
        with ExitStack() as stack:
            def p(a, b):
                stack.enter_context(patch(a, b, create=True))
            # Pretend to be on Windows
            p("colorama.ansitowin32.os.name", "nt")
            p("colorama.ansitowin32.winapi_test", lambda: True)
            p("colorama.win32.winapi_test", lambda: True)
            p("colorama.winterm.win32.windll", "non-None")
            p("colorama.winterm.get_osfhandle", lambda _: 1234)

            # Pretend that our mock stream has native ANSI support
            p(
                "colorama.winterm.win32.GetConsoleMode",
                lambda _: ENABLE_VIRTUAL_TERMINAL_PROCESSING,
            )
            SetConsoleMode = Mock()
            p("colorama.winterm.win32.SetConsoleMode", SetConsoleMode)

            stdout = Mock()
            stdout.closed = False
            stdout.isatty.return_value = True
            stdout.fileno.return_value = 1

            # Our fake console says it has native vt support, so AnsiToWin32 should
            # enable that support and do nothing else.
            stream = AnsiToWin32(stdout)
            SetConsoleMode.assert_called_with(1234, ENABLE_VIRTUAL_TERMINAL_PROCESSING)
            self.assertFalse(stream.strip)
            self.assertFalse(stream.convert)
            self.assertFalse(stream.should_wrap())

            # Now let's pretend we're on an old Windows console, that doesn't have
            # native ANSI support.
            p("colorama.winterm.win32.GetConsoleMode", lambda _: 0)
            SetConsoleMode = Mock()
            p("colorama.winterm.win32.SetConsoleMode", SetConsoleMode)

            stream = AnsiToWin32(stdout)
            SetConsoleMode.assert_called_with(1234, ENABLE_VIRTUAL_TERMINAL_PROCESSING)
            self.assertTrue(stream.strip)
            self.assertTrue(stream.convert)
            self.assertTrue(stream.should_wrap())


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from abc import ABCMeta, abstractmethod

from typing import List  # noqa
from datetime import datetime  # noqa
from _Order import Order  # noqa


class OrderRepository(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_order(self, initial_date, end_date, pos, operator):
        # type: (datetime, datetime, int, int) -> List[Order]
        raise NotImplementedError()



from ._GetBenefitApplier import (
    fill_benefit_appliers_repositories,
    find_benefit_applier,
    find_benefit_id_by_custom_property,
    find_order_benefits_id,
    get_originator_pos_id,
    get_benefit_applier_by_name,
)



# COMMENT HERE
import json


class AverageTicketReport(object):
    def __init__(self, table_repository, order_repository):
        self.table_repository = table_repository
        self.order_repository = order_repository

    def generate_tickets_report(self, business_period):
        return self._generate_tickets_report(business_period)

    def _generate_tickets_report(self, business_period):
        user_table_orders = self.table_repository.get_paid_ticket(business_period)
        user_orders = self.order_repository.get_paid_orders(business_period.replace("-", ""))

        average_ticket_report = {}
        for table_order in user_table_orders:
            user = table_order[0]
            quantity = int(table_order[1])
            average_ticket_report[user] = quantity

        for order in user_orders:
            if order[0] is None:
                continue

            user = order[0].split("user=")[1].split(",")[0]
            quantity = int(order[1])
            if user in average_ticket_report:
                average_ticket_report[user] += quantity
                continue
            average_ticket_report[user] = quantity

        report = json.dumps(average_ticket_report, default=lambda o: o.__dict__, sort_keys=True)
        report_bytes = report.encode("utf-8")

        return report_bytes



# Copyright 2009-2017 Wander Lairson Costa
# Copyright 2009-2020 PyUSB contributors
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import usb.core as core
import usb.util as util
import usb._objfinalizer as _objfinalizer
import usb.control as control

from itertools import groupby

__author__ = 'Wander Lairson Costa'

USBError = core.USBError

CLASS_AUDIO = 1
CLASS_COMM = 2
CLASS_DATA = 10
CLASS_HID = 3
CLASS_HUB = 9
CLASS_MASS_STORAGE = 8
CLASS_PER_INTERFACE = 0
CLASS_PRINTER = 7
CLASS_VENDOR_SPEC = 255
DT_CONFIG = 2
DT_CONFIG_SIZE = 9
DT_DEVICE = 1
DT_DEVICE_SIZE = 18
DT_ENDPOINT = 5
DT_ENDPOINT_AUDIO_SIZE = 9
DT_ENDPOINT_SIZE = 7
DT_HID = 33
DT_HUB = 41
DT_HUB_NONVAR_SIZE = 7
DT_INTERFACE = 4
DT_INTERFACE_SIZE = 9
DT_PHYSICAL = 35
DT_REPORT = 34
DT_STRING = 3
ENDPOINT_ADDRESS_MASK = 15
ENDPOINT_DIR_MASK = 128
ENDPOINT_IN = 128
ENDPOINT_OUT = 0
ENDPOINT_TYPE_BULK = 2
ENDPOINT_TYPE_CONTROL = 0
ENDPOINT_TYPE_INTERRUPT = 3
ENDPOINT_TYPE_ISOCHRONOUS = 1
ENDPOINT_TYPE_MASK = 3
ERROR_BEGIN = 500000
MAXALTSETTING = 128
MAXCONFIG = 8
MAXENDPOINTS = 32
MAXINTERFACES = 32
RECIP_DEVICE = 0
RECIP_ENDPOINT = 2
RECIP_INTERFACE = 1
RECIP_OTHER = 3
REQ_CLEAR_FEATURE = 1
REQ_GET_CONFIGURATION = 8
REQ_GET_DESCRIPTOR = 6
REQ_GET_INTERFACE = 10
REQ_GET_STATUS = 0
REQ_SET_ADDRESS = 5
REQ_SET_CONFIGURATION = 9
REQ_SET_DESCRIPTOR = 7
REQ_SET_FEATURE = 3
REQ_SET_INTERFACE = 11
REQ_SYNCH_FRAME = 12
TYPE_CLASS = 32
TYPE_RESERVED = 96
TYPE_STANDARD = 0
TYPE_VENDOR = 64

class Endpoint(object):
    r"""Endpoint descriptor object."""
    def __init__(self, ep):
        self.address = ep.bEndpointAddress
        self.interval = ep.bInterval
        self.maxPacketSize = ep.wMaxPacketSize
        self.type = util.endpoint_type(ep.bmAttributes)

class Interface(object):
    r"""Interface descriptor object."""
    def __init__(self, intf):
        self.alternateSetting = intf.bAlternateSetting
        self.interfaceNumber = intf.bInterfaceNumber
        self.iInterface = intf.iInterface
        self.interfaceClass = intf.bInterfaceClass
        self.interfaceSubClass = intf.bInterfaceSubClass
        self.interfaceProtocol = intf.bInterfaceProtocol
        self.endpoints = [Endpoint(e) for e in intf]

class Configuration(object):
    r"""Configuration descriptor object."""
    def __init__(self, cfg):
        self.iConfiguration = cfg.iConfiguration
        self.maxPower = cfg.bMaxPower << 1
        self.remoteWakeup = (cfg.bmAttributes >> 5) & 1
        self.selfPowered = (cfg.bmAttributes >> 6) & 1
        self.totalLength = cfg.wTotalLength
        self.value = cfg.bConfigurationValue
        self.interfaces = [
                            list(g) for k, g in groupby(
                                    sorted(
                                        [Interface(i) for i in cfg],
                                        key=lambda i: i.interfaceNumber
                                    ),
                                    lambda i: i.alternateSetting)
                        ]

class DeviceHandle(_objfinalizer.AutoFinalizedObject):
    def __init__(self, dev):
        self.dev = dev
        self.__claimed_interface = None

    def _finalize_object(self):
        util.dispose_resources(self.dev)
        self.dev = None

    def bulkWrite(self, endpoint, buffer, timeout = 100):
        r"""Perform a bulk write request to the endpoint specified.

            Arguments:
                endpoint: endpoint number.
                buffer: sequence data buffer to write.
                        This parameter can be any sequence type.
                timeout: operation timeout in milliseconds. (default: 100)
            Returns the number of bytes written.
        """
        return self.dev.write(endpoint, buffer, timeout)

    def bulkRead(self, endpoint, size, timeout = 100):
        r"""Performs a bulk read request to the endpoint specified.

            Arguments:
                endpoint: endpoint number.
                size: number of bytes to read.
                timeout: operation timeout in milliseconds. (default: 100)
            Returns a tuple with the data read.
        """
        return self.dev.read(endpoint, size, timeout)

    def interruptWrite(self, endpoint, buffer, timeout = 100):
        r"""Perform a interrupt write request to the endpoint specified.

            Arguments:
                endpoint: endpoint number.
                buffer: sequence data buffer to write.
                        This parameter can be any sequence type.
                timeout: operation timeout in milliseconds. (default: 100)
            Returns the number of bytes written.
        """
        return self.dev.write(endpoint, buffer, timeout)

    def interruptRead(self, endpoint, size, timeout = 100):
        r"""Performs a interrupt read request to the endpoint specified.

            Arguments:
                endpoint: endpoint number.
                size: number of bytes to read.
                timeout: operation timeout in milliseconds. (default: 100)
            Returns a tuple with the data read.
        """
        return self.dev.read(endpoint, size, timeout)

    def controlMsg(self, requestType, request, buffer, value = 0, index = 0, timeout = 100):
        r"""Perform a control request to the default control pipe on a device.

        Arguments:
            requestType: specifies the direction of data flow, the type
                         of request, and the recipient.
            request: specifies the request.
            buffer: if the transfer is a write transfer, buffer is a sequence
                    with the transfer data, otherwise, buffer is the number of
                    bytes to read.
            value: specific information to pass to the device. (default: 0)
                   index: specific information to pass to the device. (default: 0)
            timeout: operation timeout in milliseconds. (default: 100)
        Returns the number of bytes written.
        """
        return self.dev.ctrl_transfer(
                    requestType,
                    request,
                    wValue = value,
                    wIndex = index,
                    data_or_wLength = buffer,
                    timeout = timeout)

    def clearHalt(self, endpoint):
        r"""Clears any halt status on the specified endpoint.

        Arguments:
            endpoint: endpoint number.
        """
        self.dev.clear_halt(endpoint)

    def claimInterface(self, interface):
        r"""Claims the interface with the Operating System.

        Arguments:
            interface: interface number or an Interface object.
        """
        if isinstance(interface, Interface):
            interface = interface.interfaceNumber

        util.claim_interface(self.dev, interface)
        self.__claimed_interface = interface

    def releaseInterface(self):
        r"""Release an interface previously claimed with claimInterface."""
        util.release_interface(self.dev, self.__claimed_interface)
        self.__claimed_interface = -1

    def reset(self):
        r"""Reset the specified device by sending a RESET
            down the port it is connected to."""
        self.dev.reset()

    def resetEndpoint(self, endpoint):
        r"""Reset all states for the specified endpoint.

        Arguments:
            endpoint: endpoint number.
        """
        self.clearHalt(endpoint)

    def setConfiguration(self, configuration):
        r"""Set the active configuration of a device.

        Arguments:
            configuration: a configuration value or a Configuration object.
        """
        if isinstance(configuration, Configuration):
           configuration = configuration.value

        self.dev.set_configuration(configuration)

    def setAltInterface(self, alternate):
        r"""Sets the active alternate setting of the current interface.

        Arguments:
            alternate: an alternate setting number or an Interface object.
        """
        if isinstance(alternate, Interface):
           alternate = alternate.alternateSetting

        self.dev.set_interface_altsetting(self.__claimed_interface, alternate)

    def getString(self, index, length, langid = None):
        r"""Retrieve the string descriptor specified by index
            and langid from a device.

        Arguments:
            index: index of descriptor in the device.
            length: number of bytes of the string (ignored)
            langid: Language ID. If it is omitted, the first
                    language will be used.
        """
        return util.get_string(self.dev, index, langid).encode('ascii')

    def getDescriptor(self, desc_type, desc_index, length, endpoint = -1):
        r"""Retrieves a descriptor from the device identified by the type
        and index of the descriptor.

        Arguments:
            desc_type: descriptor type.
            desc_index: index of the descriptor.
            len: descriptor length.
            endpoint: ignored.
        """
        return control.get_descriptor(self.dev, length, desc_type, desc_index)

    def detachKernelDriver(self, interface):
        r"""Detach a kernel driver from the interface (if one is attached,
            we have permission and the operation is supported by the OS)

        Arguments:
            interface: interface number or an Interface object.
        """
        if isinstance(interface, Interface):
            interface = interface.interfaceNumber

        self.dev.detach_kernel_driver(interface)

class Device(object):
    r"""Device descriptor object"""
    def __init__(self, dev):
        self.deviceClass = dev.bDeviceClass
        self.deviceSubClass = dev.bDeviceSubClass
        self.deviceProtocol = dev.bDeviceProtocol
        self.deviceVersion = str((dev.bcdDevice >> 12) & 0xf) + \
                            str((dev.bcdDevice >> 8) & 0xf) + \
                            '.' + \
                            str((dev.bcdDevice >> 4) & 0xf) + \
                            str(dev.bcdDevice & 0xf)
        self.devnum = dev.address
        self.filename = ''
        self.iManufacturer = dev.iManufacturer
        self.iProduct = dev.iProduct
        self.iSerialNumber = dev.iSerialNumber
        self.idProduct = dev.idProduct
        self.idVendor = dev.idVendor
        self.maxPacketSize = dev.bMaxPacketSize0
        self.usbVersion = str((dev.bcdUSB >> 12) & 0xf) + \
                         str((dev.bcdUSB >> 8) & 0xf) + \
                         '.' + \
                         str((dev.bcdUSB >> 4) & 0xf) + \
                         str(dev.bcdUSB & 0xf)
        self.configurations = [Configuration(c) for c in dev]
        self.dev = dev

    def open(self):
        r"""Open the device for use.

        Returns a DeviceHandle object
        """
        return DeviceHandle(self.dev)

class Bus(object):
    r"""Bus object."""
    def __init__(self, devices):
        self.dirname = ''
        self.devices = [Device(d) for d in devices]
        self.location = self.devices[0].dev.bus

def busses():
    r"""Returns a tuple with the usb busses."""
    return (Bus(g) for k, g in groupby(
            sorted(core.find(find_all=True), key=lambda d: d.bus),
            lambda d: d.bus))




from logging import Logger

from flask import Response
from mebuhi.event import SubscriberCreator


class StartListenProcessor(object):
    def __init__(self, subscriber, logger):
        # type: (SubscriberCreator, Logger) -> None
        self.subscriber = subscriber
        self.logger = logger

    def get_caller_ip(self, request):
        if not request.headers.getlist("X-Forwarded-For"):
            return request.remote_addr
        return request.headers.getlist("X-Forwarded-For")[0]

    def handle(self, path_parameters, request):
        caller_ip = self.get_caller_ip(request)
        self.logger.info("IP: {} Start handling Start".format(caller_ip))
        sync_id = None
        try:
            subject = request.args['subject'].encode("utf-8")
            sync_id = self.subscriber.create_subscriber(subject)
            return Response(status=200, headers={"x-sync-id": sync_id})
        finally:
            log_message = "IP: {} Finish handling Start - {}"
            self.logger.info(log_message.format(caller_ip, sync_id))



from logging import Logger

from interactor.route.loyalty.search_customer_info.dto import SearchCustomerInfoRequest
from model.config import Config
from model.repository import PosRepositoryModel
from model.services import LoyaltyServiceModel
from validation_util import (
    check_if_business_day_is_opened,
    check_if_operator_is_logged,
    validate_api_key,
    check_if_order_is_in_progress,
)

from typing import Dict


class SearchCustomerInfoInteractor(object):
    def __init__(
        self,
        logger,             # type: Logger
        config,             # type: Config
        pos_repository,     # type: PosRepositoryModel
        loyalty_service,    # type: LoyaltyServiceModel
    ):
        # type: (...) -> None

        self.logger = logger
        self.config = config
        self.pos_repository = pos_repository
        self.loyalty_service = loyalty_service

    def execute(
        self,
        request     # type: SearchCustomerInfoRequest
    ):
        # type: (...) -> Dict

        pos_id = validate_api_key(
            request=request,
            secure_api_key_list=self.config.api_key_mapping,
        )

        model = self.pos_repository.get_model(pos_id)
        pos_ot = self.pos_repository.get_pos_ot(
            model=model,
        )
        response = self.loyalty_service.get_customer_info(
            pos_ot=pos_ot,
            customer_id=request.customer_id,
        )

        return response



from typing import Optional


class TableOrderSaleLine(object):

    def __init__(
        self,
        product_name,                   # type: str
        price,                          # type: float
        qty,                            # type: float
        default_qty,                    # type: Optional[int]
        item_type,                      # type: str
        seat,                           # type: Optional[int]
        level,                          # type: int
        comment,                        # type: Optional[str]
        line_number,                    # type: int
        service_period,                 # type: Optional[str]
        order_id=None,                  # type: Optional[int]
        item_id=None,                   # type: Optional[str]
        part_code=None,                 # type: Optional[int]
        overwritten_unit_price=None,    # type: Optional[float]
        fraction_parent=None,           # type: Optional[int]
    ):
        # type: (...) -> None

        self.product_name = product_name
        self.price = price
        self.qty = qty
        self.default_qty = default_qty
        self.item_type = item_type
        self.seat = seat
        self.level = level
        self.comment = comment
        self.line_number = line_number
        self.service_period = service_period
        self.order_id = order_id
        self.item_id = item_id
        self.part_code = part_code
        self.overwritten_unit_price = overwritten_unit_price
        self.fraction_parent = fraction_parent



# coding: utf-8
from __future__ import unicode_literals, division, absolute_import, print_function


def fill_width(bytes_, width):
    """
    Ensure a byte string representing a positive integer is a specific width
    (in bytes)

    :param bytes_:
        The integer byte string

    :param width:
        The desired width as an integer

    :return:
        A byte string of the width specified
    """

    while len(bytes_) < width:
        bytes_ = b'\x00' + bytes_
    return bytes_



# encoding: utf-8

from typing import (
    Dict,
    List,
    Tuple
)

from benefitsreport.repository import BenefitsOrderRepository


class BenefitsReport(object):
    def __init__(self, order_repository):
        # type: (BenefitsOrderRepository) -> None

        self.order_repository = order_repository

    def get_benefits_orders_detailed(
            self,
            initial_date,   # type: str
            end_date,       # type: str
            operator_id,    # type: str
            benefits        # type: Dict
    ):
        # type: (...) -> Dict[str, list]

        orders = self.order_repository.get_benefits_orders_detailed(
            initial_date,
            end_date,
            operator_id,
            benefits
        )

        report_data = {}
        for item in orders:
            benefit = {
                "order_id": item["order_id"],
                "ordered_qty": item["ordered_qty"],
                "product_unit_price": item["product_unit_price"],
                "product_name": item["product_name"],
                "benefit_total_value": item["benefit_total_value"],
                "order_time": item["order_time"],
                "operator": item["operator"],
                "discount_type": item["discount_type"],
                "total_gross": item["total_gross"]
            }
            benefits_for_period = report_data.setdefault(item["business_period"], [])
            benefits_for_period.append(benefit)

        return report_data

    def get_benefits_orders_consolidated(
            self,
            initial_date,  # type: str
            end_date,  # type: str
            operator_id  # type: str
    ):
        # type: (...) -> Dict[str, Tuple[int, float, float, str]]

        def merge_benefits(
                benefits  # type: List[Dict[str, Dict]]
        ):
            # type: (...) -> Dict[str, Tuple[int, float, float, str]]

            i_quantity = 0
            i_product_unit_price = 1
            i_benefit_total_value = 2

            merged = {}
            for benefit_data in benefits:
                for benefit_id, benefit in benefit_data.items():
                    if benefit["quantity"] > 0.0:
                        merged.setdefault(
                            benefit_id,
                            [
                                0,  # i_quantity
                                0.0,  # i_product_unit_price
                                0.0,  # i_benefit_total_value
                                benefit["benefit_user_description"],
                            ]
                        )
                        merged[benefit_id][i_quantity] += benefit["quantity"]
                        merged[benefit_id][i_benefit_total_value] += benefit["benefit_total_value"]
                        merged[benefit_id][i_product_unit_price] += (
                            benefit["quantity"] * benefit["product_unit_price"]
                        )

            return {k: tuple(v) for k, v in merged.items()}

        report_data = merge_benefits(
            self.order_repository.get_benefits_orders_consolidated(
                initial_date,
                end_date,
                operator_id
            )
        )

        return report_data



from messagebus import TokenCreator, TokenPriority


class ResponseToken(object):
    NCOUPON_MESSAGE_GROUP = "113"

    TK_NCOUPON_ERROR = TokenCreator.create_token(TokenPriority.low, NCOUPON_MESSAGE_GROUP, "F001")



# -*- coding: utf-8 -*-
import time
from cStringIO import StringIO
from decimal import Decimal as D
# Our modules
from systools import sys_log_error
from reports import mbcontext
from msgbus import TK_POS_GETPOSLIST, TK_SYS_NAK
from persistence import Driver as DBDriver

from utils.report_formatters import translate_pos_label


COLS = 38
SEPARATOR = ("=" * COLS)
SINGLE_SEPARATOR = ("-" * COLS)
DATE_TIME_FMT = "%d/%m/%Y %H:%M:%S"
DATE_FMT = "%d/%m/%Y"
ZERO = D("0.00")


def _cut(s):
    return s[:COLS] if (len(s) > COLS) else s


def _center(s):
    s = _cut(s)
    miss = COLS - len(s)
    if miss == 0:
        return s
    left = miss / 2
    return (" " * left) + s


def _join(dic1, dic2):
    d = dict(dic1)
    d.update(dic2)
    return d


def diffdates(d1, d2):
    # Date format: %Y-%m-%d %H:%M:%S
    dt1, _, us = d1.partition(".")
    dt2, _, us = d2.partition(".")
    return (time.mktime(time.strptime(dt2, "%Y-%m-%dT%H:%M:%S")) - time.mktime(time.strptime(dt1, "%Y-%m-%dT%H:%M:%S")))


def speedOfService(_, period, pos, store_id="", *args):
    if pos == '0':
        # get a pos list
        msg = mbcontext.MB_EasySendMessage("PosController", TK_POS_GETPOSLIST)
        if msg.token == TK_SYS_NAK:
            sys_log_error("Could not retrieve PosList")
            raise Exception("Could not retrieve PosList")

        pos_list = sorted(map(int, msg.data.split("\0")))
    else:
        pos_list = [pos]

    slot = {}
    cont = {}

    pos_error_list = []
    for pos_id in pos_list:
        orders = {}
        conn = None
        try:
            # Opens the database connection
            conn = DBDriver().open(mbcontext, service_name="Persistence", dbname=str(pos_id))

            # Get the data
            orig_id = "POS%04d" % int(pos_id)
            cursor = conn.select('''
SELECT Orders.Orderid AS OrderId, SessionId,
CASE WHEN OrderStateHistory.Stateid IN (1, 6) THEN "START" WHEN OrderStateHistory.Stateid IN (2, 4, 5) THEN "END" END AS State,
CASE WHEN OrderStateHistory.Stateid IN (1, 6) THEN MIN(Timestamp) ELSE MAX(Timestamp) END AS Time
FROM OrderStateHistory
JOIN Orders ON Orders.OrderId = OrderStateHistory.OrderId
WHERE OrderStateHistory.StateId IN (1, 2, 4, 5, 6) AND
BusinessPeriod = '%(period)s' AND
OriginatorId = '%(orig_id)s'
GROUP BY Orders.Orderid, OrderStateHistory.Stateid''' % locals())

            # TODO: Quando uma venda for InProgess->Stored->Recalled->Paid estamos contando apenas Recalled->Paid
            for row in cursor:
                order_id, session_id, state, timestamp = map(row.get_entry, ("OrderId", "SessionId", "State", "Time"))

                user_id = session_id.split(",")[1].replace("user=", "")
                if order_id not in orders:
                    orders[order_id] = {}
                orders[order_id]['USER'] = user_id

                if state == "START":
                    orders[order_id]['START'] = user_id = timestamp
                if state == "END":
                    orders[order_id]['END'] = user_id = timestamp
        except Exception as e:
            pos_error_list.append(pos_id)
            sys_log_error("Error openning database for POS %s - %s" % (pos_id, e.message))
        finally:
            if conn:
                # Close database connection
                conn.close()

        for key in orders:
            order = orders[key]
            if 'START' in order and 'END' in order:
                seconds = diffdates(order['START'], order['END'])

                user = order['USER']
                if user in slot.keys():
                    slot[user] += seconds
                    cont[user] += 1
                else:
                    slot[user] = seconds
                    cont[user] = 1

    slot_total = 0
    cont_total = 0
    for operator in slot.keys():
        slot_total += slot[operator]
        cont_total += cont[operator]

        tmp = slot[operator] / cont[operator]
        slot[operator] = tmp

    if cont_total != 0:
        tmp = slot_total / cont_total
    else:
        tmp = 0

    slot_total = tmp

    # create string I/O to append the report info
    report = StringIO()
    if not report:
        return

    #         11        21        31      |
    # 12345678901234567890123456789012345678
    #
    # ======================================
    #       Relatorio Speed of Service
    #   Data/hora.....: 03/26/2009 13:06:12
    #   Dia Util......: 03/26/2009
    #   POS ..........: 01
    # ======================================
    # Operador                         Tempo
    # --------------------------------------
    # 326                               0:00
    # ======================================

    store_id = store_id
    # titulo
    if pos == '0':
        pos_list = "Todos"
    title = _center("Relatorio Speed of Service")
    current_datetime = time.strftime(DATE_TIME_FMT)
    business_period = "%02d/%02d/%04d" % (int(period[6:8]), int(period[4:6]), int(period[:4]))
    pos_label = translate_pos_label(pos_id=pos)
    report.write(
        """%(SEPARATOR)s
%(title)s
  Loja..........: %(store_id)s
  Data/hora.....: %(current_datetime)s
  Dia Util......: %(business_period)s
  %(pos_label)s...........: %(pos_list)s
""" % _join(globals(), locals()))
    if len(pos_error_list) > 0:
        report.write(("  {} Erro......: %s\n" % pos_error_list).format(pos_label))
    report.write("%s\n" % SEPARATOR)
    report.write("Operador         Vendas          Tempo\n")
    report.write("%s\n" % SINGLE_SEPARATOR)

    # imprime as linhas do relatorio
    for operator in slot.keys():
        report.write("%s" % operator)

        width = 23 - len(operator)
        tmp = str(cont[operator])
        report.write("%s" % tmp.rjust(width))

        width = 15
        tmp = time.strftime('%H:%M:%S', time.gmtime(slot[operator]))
        report.write("%s\n" % tmp.rjust(width))

    report.write("%s\n" % SEPARATOR)

    report.write("TOTAL")
    width = 23 - len("TOTAL")
    tmp = str(cont_total)
    report.write("%s" % tmp.rjust(width))
    width = 15
    tmp = time.strftime('%H:%M:%S', time.gmtime(slot_total))
    report.write("%s\n" % tmp.rjust(width))

    report.write("%s\n" % SEPARATOR)

    return report.getvalue()



from ._GetAndLockCouponInteractor import GetAndLockCouponInteractor
from ._GetLoyaltyCustomerInfoInteractor import GetLoyaltyCustomerInfoInteractor
from ._LoyaltyAvailableInteractor import LoyaltyAvailableInteractor
from ._LoyaltyIdentifierInteractor import LoyaltyIdentifierInteractor
from ._LockBenefitInteractor import LockBenefitInteractor



from ._TableReportHeaderDto import TableReportHeaderDto  # noqa
from ._TableReportBodyDto import TableReportBodyDto  # noqa


class TableReportDto(object):

    def __init__(self, header, body):
        # type: (TableReportHeaderDto, TableReportBodyDto) -> None
        self.header = header
        self.body = body



"""
Contains information about the dateutil version.
"""

VERSION_MAJOR = 2
VERSION_MINOR = 6
VERSION_PATCH = 1

VERSION_TUPLE = (VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH)
VERSION = '.'.join(map(str, VERSION_TUPLE))



from xml.etree import cElementTree as eTree

from application.service import OrderService, BenefitService
from application.model import Configurations
from typing import Dict, List, Optional


class OnOrderReopenInteractor(object):

    def __init__(
        self,
        configs,                        # type: Configurations
        order_service,                  # type: OrderService
        benefit_service,                # type: BenefitService
        include_table_orders=False,     # type: Optional[bool]
    ):
        # type: (...) -> None

        self.configs = configs
        self.order_service = order_service
        self.benefit_service = benefit_service
        self.include_table_orders = include_table_orders

    def execute(
        self,
        pos_id,         # type: int
        table_id=None,  # type: Optional[str]
    ):
        # type: (...) -> Optional[eTree.XML]

        static_uow = self.benefit_service.get_static_unit_of_work(
            pos_id=pos_id,
            table_id=table_id,
        )

        static_uow.pos_ot.blkopnotify = True
        order_picture = static_uow.order_picture
        order = self.order_service.get_order(
            pos_id=pos_id,
            model=static_uow.model,
            create_order=False,
            order_picture=order_picture,
            static_uow=static_uow,
        )
        if self.include_table_orders is False and order.table_id is not None:
            return

        if self.include_table_orders is True and order.table_id is None:
            return

        state = order_picture.get("state") or "" if order_picture else ""
        if state != "IN_PROGRESS":
            return order_picture

        order = self.order_service.get_order(
            pos_id=pos_id,
            order_picture=order_picture,
            static_uow=static_uow,
        )
        order_picture = order.order_picture

        order_picture = self.benefit_service.save_persistent_benefits(
            pos_id=pos_id,
            order_picture=order_picture,
        )
        static_uow.set_order_picture(order_picture=order_picture)

        order_benefits = self.order_service.get_applied_benefits_from_order_picture(
            pos_id=pos_id,
            order_picture=order_picture,
        )
        if not order_benefits:
            return order_picture

        benefit_pre_total_sequence = self.order_service.get_benefit_pre_total_sequence_from_order_picture(
            pos_id=pos_id,
            order_picture=order_picture,
        )
        benefits_to_remove = self._get_benefits_to_remove(
            order_benefits,
            benefit_pre_total_sequence
        )
        if len(benefits_to_remove) > 0:
            self.benefit_service.remove_benefits(
                pos_id=pos_id,
                benefit_keys=benefits_to_remove,
                static_uow=static_uow,
            )
            return static_uow.order_picture

    def _get_benefits_to_remove(
        self,                           # type: OnOrderReopenInteractor
        applied_benefits,               # type: List
        benefit_pre_total_sequence,     # type: List
    ):
        # type: (...) -> List

        benefits_to_remove = []
        for applied_benefit in applied_benefits:
            benefit_key_to_remove = self._get_benefit_id_to_remove(
                applied_benefit=applied_benefit,
                benefit_pre_total_sequence=benefit_pre_total_sequence
            )
            if benefit_key_to_remove:
                benefits_to_remove.append(benefit_key_to_remove)

        return benefits_to_remove

    @staticmethod
    def _get_benefit_id_to_remove(
        applied_benefit,                # type: Dict
        benefit_pre_total_sequence,     # type: Dict
    ):
        # type: (...) -> Optional[int]

        benefit_applied = applied_benefit.get("benefit_applied")
        if not benefit_applied:
            return None

        benefit = applied_benefit.get("benefit", {})
        is_loyalty_benefit = benefit.get("is_loyalty_benefit", False)
        if (
            applied_benefit.get("benefit_key", "") in benefit_pre_total_sequence
            and benefit.get("automatic", "") != "ON_TOTAL"
            and not is_loyalty_benefit
            and benefit.get("persistent", True)
        ):
            return None

        return applied_benefit.get("benefit_key")



from __future__ import absolute_import
import sys

if sys.version_info[0] < 3:
    from future.moves.html import *
else:
    raise ImportError('This package should not be accessible on Python 3. '
                      'Either you are trying to run from the python-future src folder '
                      'or your installation of python-future is corrupted.')



from abc import ABCMeta, abstractmethod


class Authenticator(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def authenticate(self, username, password):
        # type: (str, str) -> None
        raise NotImplementedError()



from typing import Optional


class Event(object):
    def __init__(self, subject, event_type=None, data=None, source=0, synchronous=False, timeout=-1):
        # type: (unicode, Optional[unicode], Optional[bytes], int, bool, int) -> None
        self.subject = subject
        self.event_type = event_type
        self.source = source
        self.data = data
        self.synchronous = synchronous
        self.timeout = timeout
        self.imp_message = None



# encoding: utf-8

from typing import Any


class JustificationDiscount(object):
    def __init__(
        self,
        row_database,               # type: Any
        benefit_id,                 # type: str
        justification,              # type: str
        applied_by,                 # type: str
        benefit_total_value,        # type: float
    ):
        # type: (...) -> None

        self.order_id = row_database.get_entry("OrderId")
        self.order_date = row_database.get_entry("OrderDate")
        self.order_time = row_database.get_entry("OrderTime")
        self.order_value = row_database.get_entry("TotalGross")
        self.benefit_id = benefit_id
        self.justification = justification
        self.applied_by = applied_by
        self.benefit_total_value = benefit_total_value



from typing import Optional
from xml.etree import cElementTree as eTree

from application.repository.pos.tablemgr import TablePictureRepository


class TableService(object):
    def __init__(
        self,                           # type: TableService
        table_picture_repository,       # type: TablePictureRepository
    ):
        # type: (...) -> None

        self.table_picture_repository = table_picture_repository

    def get_table_picture(
        self,                   # type: TableService
        pos_id,                 # type: int
        order_picture,          # type: eTree
    ):
        # type: (...) -> Optional[eTree]

        table_id = None
        table_id_key = order_picture.findall(".//OrderProperty[@key='TABLE_ID']")
        if table_id_key:
            table_id = table_id_key[0].get("value")

        if table_id:
            return self.table_picture_repository.get_table_info(
                pos_id=pos_id,
                table_id=table_id
            )



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.

from __future__ import absolute_import

import distutils.util

try:
    from importlib.machinery import EXTENSION_SUFFIXES
except ImportError:  # pragma: no cover
    import imp

    EXTENSION_SUFFIXES = [x[0] for x in imp.get_suffixes()]
    del imp
import platform
import re
import sys
import sysconfig
import warnings


INTERPRETER_SHORT_NAMES = {
    "python": "py",  # Generic.
    "cpython": "cp",
    "pypy": "pp",
    "ironpython": "ip",
    "jython": "jy",
}


_32_BIT_INTERPRETER = sys.maxsize <= 2 ** 32


class Tag(object):

    __slots__ = ["_interpreter", "_abi", "_platform"]

    def __init__(self, interpreter, abi, platform):
        self._interpreter = interpreter.lower()
        self._abi = abi.lower()
        self._platform = platform.lower()

    @property
    def interpreter(self):
        return self._interpreter

    @property
    def abi(self):
        return self._abi

    @property
    def platform(self):
        return self._platform

    def __eq__(self, other):
        return (
            (self.platform == other.platform)
            and (self.abi == other.abi)
            and (self.interpreter == other.interpreter)
        )

    def __hash__(self):
        return hash((self._interpreter, self._abi, self._platform))

    def __str__(self):
        return "{}-{}-{}".format(self._interpreter, self._abi, self._platform)

    def __repr__(self):
        return "<{self} @ {self_id}>".format(self=self, self_id=id(self))


def parse_tag(tag):
    tags = set()
    interpreters, abis, platforms = tag.split("-")
    for interpreter in interpreters.split("."):
        for abi in abis.split("."):
            for platform_ in platforms.split("."):
                tags.add(Tag(interpreter, abi, platform_))
    return frozenset(tags)


def _normalize_string(string):
    return string.replace(".", "_").replace("-", "_")


def _cpython_interpreter(py_version):
    # TODO: Is using py_version_nodot for interpreter version critical?
    return "cp{major}{minor}".format(major=py_version[0], minor=py_version[1])


def _cpython_abis(py_version):
    abis = []
    version = "{}{}".format(*py_version[:2])
    debug = pymalloc = ucs4 = ""
    with_debug = sysconfig.get_config_var("Py_DEBUG")
    has_refcount = hasattr(sys, "gettotalrefcount")
    # Windows doesn't set Py_DEBUG, so checking for support of debug-compiled
    # extension modules is the best option.
    # https://github.com/pypa/pip/issues/3383#issuecomment-173267692
    has_ext = "_d.pyd" in EXTENSION_SUFFIXES
    if with_debug or (with_debug is None and (has_refcount or has_ext)):
        debug = "d"
    if py_version < (3, 8):
        with_pymalloc = sysconfig.get_config_var("WITH_PYMALLOC")
        if with_pymalloc or with_pymalloc is None:
            pymalloc = "m"
        if py_version < (3, 3):
            unicode_size = sysconfig.get_config_var("Py_UNICODE_SIZE")
            if unicode_size == 4 or (
                unicode_size is None and sys.maxunicode == 0x10FFFF
            ):
                ucs4 = "u"
    elif debug:
        # Debug builds can also load "normal" extension modules.
        # We can also assume no UCS-4 or pymalloc requirement.
        abis.append("cp{version}".format(version=version))
    abis.insert(
        0,
        "cp{version}{debug}{pymalloc}{ucs4}".format(
            version=version, debug=debug, pymalloc=pymalloc, ucs4=ucs4
        ),
    )
    return abis


def _cpython_tags(py_version, interpreter, abis, platforms):
    for abi in abis:
        for platform_ in platforms:
            yield Tag(interpreter, abi, platform_)
    for tag in (Tag(interpreter, "abi3", platform_) for platform_ in platforms):
        yield tag
    for tag in (Tag(interpreter, "none", platform_) for platform_ in platforms):
        yield tag
    # PEP 384 was first implemented in Python 3.2.
    for minor_version in range(py_version[1] - 1, 1, -1):
        for platform_ in platforms:
            interpreter = "cp{major}{minor}".format(
                major=py_version[0], minor=minor_version
            )
            yield Tag(interpreter, "abi3", platform_)


def _pypy_interpreter():
    return "pp{py_major}{pypy_major}{pypy_minor}".format(
        py_major=sys.version_info[0],
        pypy_major=sys.pypy_version_info.major,
        pypy_minor=sys.pypy_version_info.minor,
    )


def _generic_abi():
    abi = sysconfig.get_config_var("SOABI")
    if abi:
        return _normalize_string(abi)
    else:
        return "none"


def _pypy_tags(py_version, interpreter, abi, platforms):
    for tag in (Tag(interpreter, abi, platform) for platform in platforms):
        yield tag
    for tag in (Tag(interpreter, "none", platform) for platform in platforms):
        yield tag


def _generic_tags(interpreter, py_version, abi, platforms):
    for tag in (Tag(interpreter, abi, platform) for platform in platforms):
        yield tag
    if abi != "none":
        tags = (Tag(interpreter, "none", platform_) for platform_ in platforms)
        for tag in tags:
            yield tag


def _py_interpreter_range(py_version):
    """
    Yield Python versions in descending order.

    After the latest version, the major-only version will be yielded, and then
    all following versions up to 'end'.
    """
    yield "py{major}{minor}".format(major=py_version[0], minor=py_version[1])
    yield "py{major}".format(major=py_version[0])
    for minor in range(py_version[1] - 1, -1, -1):
        yield "py{major}{minor}".format(major=py_version[0], minor=minor)


def _independent_tags(interpreter, py_version, platforms):
    """
    Return the sequence of tags that are consistent across implementations.

    The tags consist of:
    - py*-none-<platform>
    - <interpreter>-none-any
    - py*-none-any
    """
    for version in _py_interpreter_range(py_version):
        for platform_ in platforms:
            yield Tag(version, "none", platform_)
    yield Tag(interpreter, "none", "any")
    for version in _py_interpreter_range(py_version):
        yield Tag(version, "none", "any")


def _mac_arch(arch, is_32bit=_32_BIT_INTERPRETER):
    if not is_32bit:
        return arch

    if arch.startswith("ppc"):
        return "ppc"

    return "i386"


def _mac_binary_formats(version, cpu_arch):
    formats = [cpu_arch]
    if cpu_arch == "x86_64":
        if version < (10, 4):
            return []
        formats.extend(["intel", "fat64", "fat32"])

    elif cpu_arch == "i386":
        if version < (10, 4):
            return []
        formats.extend(["intel", "fat32", "fat"])

    elif cpu_arch == "ppc64":
        # TODO: Need to care about 32-bit PPC for ppc64 through 10.2?
        if version > (10, 5) or version < (10, 4):
            return []
        formats.append("fat64")

    elif cpu_arch == "ppc":
        if version > (10, 6):
            return []
        formats.extend(["fat32", "fat"])

    formats.append("universal")
    return formats


def _mac_platforms(version=None, arch=None):
    version_str, _, cpu_arch = platform.mac_ver()
    if version is None:
        version = tuple(map(int, version_str.split(".")[:2]))
    if arch is None:
        arch = _mac_arch(cpu_arch)
    platforms = []
    for minor_version in range(version[1], -1, -1):
        compat_version = version[0], minor_version
        binary_formats = _mac_binary_formats(compat_version, arch)
        for binary_format in binary_formats:
            platforms.append(
                "macosx_{major}_{minor}_{binary_format}".format(
                    major=compat_version[0],
                    minor=compat_version[1],
                    binary_format=binary_format,
                )
            )
    return platforms


# From PEP 513.
def _is_manylinux_compatible(name, glibc_version):
    # Check for presence of _manylinux module.
    try:
        import _manylinux

        return bool(getattr(_manylinux, name + "_compatible"))
    except (ImportError, AttributeError):
        # Fall through to heuristic check below.
        pass

    return _have_compatible_glibc(*glibc_version)


def _glibc_version_string():
    # Returns glibc version string, or None if not using glibc.
    import ctypes

    # ctypes.CDLL(None) internally calls dlopen(NULL), and as the dlopen
    # manpage says, "If filename is NULL, then the returned handle is for the
    # main program". This way we can let the linker do the work to figure out
    # which libc our process is actually using.
    process_namespace = ctypes.CDLL(None)
    try:
        gnu_get_libc_version = process_namespace.gnu_get_libc_version
    except AttributeError:
        # Symbol doesn't exist -> therefore, we are not linked to
        # glibc.
        return None

    # Call gnu_get_libc_version, which returns a string like "2.5"
    gnu_get_libc_version.restype = ctypes.c_char_p
    version_str = gnu_get_libc_version()
    # py2 / py3 compatibility:
    if not isinstance(version_str, str):
        version_str = version_str.decode("ascii")

    return version_str


# Separated out from have_compatible_glibc for easier unit testing.
def _check_glibc_version(version_str, required_major, minimum_minor):
    # Parse string and check against requested version.
    #
    # We use a regexp instead of str.split because we want to discard any
    # random junk that might come after the minor version -- this might happen
    # in patched/forked versions of glibc (e.g. Linaro's version of glibc
    # uses version strings like "2.20-2014.11"). See gh-3588.
    m = re.match(r"(?P<major>[0-9]+)\.(?P<minor>[0-9]+)", version_str)
    if not m:
        warnings.warn(
            "Expected glibc version with 2 components major.minor,"
            " got: %s" % version_str,
            RuntimeWarning,
        )
        return False
    return (
        int(m.group("major")) == required_major
        and int(m.group("minor")) >= minimum_minor
    )


def _have_compatible_glibc(required_major, minimum_minor):
    version_str = _glibc_version_string()
    if version_str is None:
        return False
    return _check_glibc_version(version_str, required_major, minimum_minor)


def _linux_platforms(is_32bit=_32_BIT_INTERPRETER):
    linux = _normalize_string(distutils.util.get_platform())
    if linux == "linux_x86_64" and is_32bit:
        linux = "linux_i686"
    manylinux_support = (
        ("manylinux2014", (2, 17)),  # CentOS 7 w/ glibc 2.17 (PEP 599)
        ("manylinux2010", (2, 12)),  # CentOS 6 w/ glibc 2.12 (PEP 571)
        ("manylinux1", (2, 5)),  # CentOS 5 w/ glibc 2.5 (PEP 513)
    )
    manylinux_support_iter = iter(manylinux_support)
    for name, glibc_version in manylinux_support_iter:
        if _is_manylinux_compatible(name, glibc_version):
            platforms = [linux.replace("linux", name)]
            break
    else:
        platforms = []
    # Support for a later manylinux implies support for an earlier version.
    platforms += [linux.replace("linux", name) for name, _ in manylinux_support_iter]
    platforms.append(linux)
    return platforms


def _generic_platforms():
    platform = _normalize_string(distutils.util.get_platform())
    return [platform]


def _interpreter_name():
    name = platform.python_implementation().lower()
    return INTERPRETER_SHORT_NAMES.get(name) or name


def _generic_interpreter(name, py_version):
    version = sysconfig.get_config_var("py_version_nodot")
    if not version:
        version = "".join(map(str, py_version[:2]))
    return "{name}{version}".format(name=name, version=version)


def sys_tags():
    """
    Returns the sequence of tag triples for the running interpreter.

    The order of the sequence corresponds to priority order for the
    interpreter, from most to least important.
    """
    py_version = sys.version_info[:2]
    interpreter_name = _interpreter_name()
    if platform.system() == "Darwin":
        platforms = _mac_platforms()
    elif platform.system() == "Linux":
        platforms = _linux_platforms()
    else:
        platforms = _generic_platforms()

    if interpreter_name == "cp":
        interpreter = _cpython_interpreter(py_version)
        abis = _cpython_abis(py_version)
        for tag in _cpython_tags(py_version, interpreter, abis, platforms):
            yield tag
    elif interpreter_name == "pp":
        interpreter = _pypy_interpreter()
        abi = _generic_abi()
        for tag in _pypy_tags(py_version, interpreter, abi, platforms):
            yield tag
    else:
        interpreter = _generic_interpreter(interpreter_name, py_version)
        abi = _generic_abi()
        for tag in _generic_tags(interpreter, py_version, abi, platforms):
            yield tag
    for tag in _independent_tags(interpreter, py_version, platforms):
        yield tag



from application.domain import TableService  # noqa


class CloseTableInteractor(object):
    def __init__(self, table_service):
        # type: (TableService) -> None
        self.table_service = table_service

    def execute(self, pos_id, table_id):
        # type: (str, str) -> str
        return self.table_service.close_table(pos_id, table_id)



# encoding: utf-8
import datetime

from old_helper import BaseRepository
from typing import (
    Any,
    Optional
)

from ..model import Order


class OrderRepository(BaseRepository):

    def __init__(self, mb_context, pos_list):
        super(OrderRepository, self).__init__(mb_context)
        self.pos_list = pos_list

    def get_sales_by_date(
        self,
        user_id,                        # type: str
        start_date,                     # type: datetime.datetime
        end_date,                       # type: datetime.datetime
        by_created_order_time=False     # type: Optional[bool]
    ):
        # type: (...) -> Any

        def inner_func(conn):
            start_date_formatted = datetime.datetime.strftime(start_date, "%Y-%m-%dT%H:%M:%S")
            end_date_formatted = datetime.datetime.strftime(end_date, "%Y-%m-%dT%H:%M:%S")
            query_operator = "AND SessionId like '%user={0},%'".format(user_id) if len(user_id) > 0 else ""

            base_query = self._OrdersCreatedTimeQuery if by_created_order_time else self._OrdersQuery
            query = base_query.format(query_operator, start_date_formatted, end_date_formatted)

            orders = []
            for x in conn.select(query):
                orders.append(Order(x.get_entry("Timestamp"), x.get_entry("SaleType"), float(x.get_entry("Amount"))))
            return orders

        return self.execute_parallel_in_all_databases_returning_flat_list(inner_func, self.pos_list)

    _OrdersQuery = """\
    SELECT ocp.Value as Timestamp, st.TypeDescr as SaleType, o.TotalGross as Amount FROM Orders o
    INNER JOIN OrderCustomProperties ocp ON o.OrderId = ocp.OrderID
    INNER JOIN SaleType st ON o.SaleType = st.TypeId
    WHERE  o.stateid = 5 
        {0}
        AND KEY = 'FISCALIZATION_DATE'
        AND ocp.value >= '{1}'
        AND ocp.value < '{2}'
    ORDER BY Timestamp
    """

    _OrdersCreatedTimeQuery = """\
    SELECT st.TypeDescr as SaleType, o.TotalGross as Amount, o.CreatedAt as Timestamp FROM Orders o
    INNER JOIN SaleType st ON o.SaleType = st.TypeId
    WHERE  o.stateid in (2, 5, 8)
        {0}
        AND (o.OrderSubType <> 'SPLIT_SERVICE' OR o.OrderSubType IS NULL)
        AND o.CreatedAt >= '{1}'
        AND o.CreatedAt < '{2}'
    ORDER BY o.CreatedAt
    """



from typing import List


class StoreGoals(object):
    def __init__(
        self,
        amount_goal,            # type: int
        average_sale_goal,      # type: int
        operator_sale_goal,     # type: int
        item_goals,             # type: List
    ):
        # type: (...) -> None

        self.amount_goal = amount_goal
        self.average_sale_goal = average_sale_goal
        self.operator_sale_goal = operator_sale_goal
        self.sold_quantity = 0
        self.item_goals = item_goals



import json

from typing import Any

from messagebus import (
    DataType,
    DefaultToken,
    Event,
    Message,
    MessageBus
)
from messageprocessorutil import UuidMessageProcessor

from application.interactor import OnTabOpenInteractor
from application.model import Configurations


class OnTabOpenProcessor(UuidMessageProcessor):
    def __init__(
            self,
            configs,    # type: Configurations
            interactor  # type: OnTabOpenInteractor
    ):
        # type: (...) -> None

        super(OnTabOpenProcessor, self).__init__()
        self.logger = configs.logger
        self.configs = configs
        self.interactor = interactor

    def parse_data(
            self,       # type: OnTabOpenProcessor
            data,       # type: str
            event_type  # type: str
    ):
        # type: (...) -> (int, int)

        data = json.loads(data)
        pos_id = int(data.get("pos_id"))
        table_id = int(data.get("table_id")[4:])
        return pos_id, table_id

    def call_business(
            self,   # type: OnTabOpenProcessor
            params  # type: tuple
    ):
        # type: (...) -> str

        pos_id, table_id = params
        return self.interactor.execute(pos_id, table_id)

    def format_response(
            self,           # type: OnTabOpenProcessor
            message_bus,    # type: MessageBus
            message,        # type: Message
            event,          # type: Event
            request,        # type: Any
            response        # type: Any
    ):
        # type: (...) -> None

        if message:
            message_bus.reply_message(
                message=message,
                reply_message=Message(
                    DefaultToken.TK_SYS_ACK,
                    DataType.string,
                    response
                )
            )

    def format_exception(
            self,           # type: OnTabOpenProcessor
            message_bus,    # type: MessageBus
            message,        # type: Message
            event,          # type: Event
            input_obj,      # type: Any
            exception       # type: Any
    ):
        # type: (...) -> None

        if message:
            message_bus.reply_message(
                message=message,
                reply_message=Message(
                    DefaultToken.TK_SYS_NAK,
                    DataType.param,
                    str(exception)
                )
            )

    def format_parse_exception(
            self,           # type: OnTabOpenProcessor
            message_bus,    # type: MessageBus
            message,        # type: Message
            event,          # type: Event
            data,           # type: Any
            exception       # type: Any
    ):
        # type: (...) -> None

        self.format_exception(
            message_bus=message_bus,
            message=message,
            event=event,
            input_obj=data,
            exception=exception,
        )
